setwd("C:/Users/Work/AAA/Programming/biology/STACEYOlaveSimulations")

article.dpath <- "C:/Users/Work/AAA/Docs/Latex/Biology/STACEY/PopModelThreeMoves/"

article.pdf <- paste0(article.dpath, "Olave-figure.pdf")
article.eps <- paste0(article.dpath, "Olave-figure.eps")

source("Olave-arrays.r")

ptrues <- list(YHg4.ptrue.clustering, ZHg4.ptrue.clustering, 
               YHg8.ptrue.clustering, ZHg8.ptrue.clustering, 
               YHg14.ptrue.clustering, ZHg14.ptrue.clustering,
               YEg4.ptrue.clustering, ZEg4.ptrue.clustering, 
               YEg8.ptrue.clustering, ZEg8.ptrue.clustering, 
               YEg14.ptrue.clustering, ZEg14.ptrue.clustering)

maxpp.dists <- list(YHg4.randdists.to.maxpp, ZHg4.randdists.to.maxpp,
                    YHg8.randdists.to.maxpp, ZHg8.randdists.to.maxpp,
                    YHg14.randdists.to.maxpp, ZHg14.randdists.to.maxpp,
                    YEg4.randdists.to.maxpp, ZEg4.randdists.to.maxpp,
                    YEg8.randdists.to.maxpp, ZEg8.randdists.to.maxpp,
                    YEg14.randdists.to.maxpp, ZEg14.randdists.to.maxpp)

olave.Iss <- list(YHg4.olave.Is, ZHg4.olave.Is,
                  YHg8.olave.Is, ZHg8.olave.Is,
                  YHg14.olave.Is, ZHg14.olave.Is,
                  YEg4.olave.Is, ZEg4.olave.Is,
                  YEg8.olave.Is, ZEg8.olave.Is,
                  YEg14.olave.Is, ZEg14.olave.Is)


cfg.names <- c("YH4","ZH4","YH8","ZH8","YH14","ZH14",
               "YE4","ZE4","YE8","ZE8","YE14","ZE14")


do.olave.figure <- function() {
  layout(mat=matrix(c(1,2),nrow=2, ncol=1), heights=c(4,3))
  
  oldpar <- par(las=3, mar=c(6,5,1,1))
  
  boxplot(ptrues,
          names=cfg.names,
          boxwex=.5,
          ylab="Posterior probabilities",
          col=rep(c("white","grey90"),4))
  
  mtext("Error count:",side=1,line=4.5,at=.3, adj=1, las=1)
  for (cfg in 1:length(cfg.names)) {
    errs <- length(which(maxpp.dists[[cfg]] > 0))
    mtext(errs,side=1,line=4.5,at=cfg, las=1)
  }
  abline(v=6.5)
  
  # The boxplots show posterior probability of true clustering
  # over 50 replicates for eight configurations YHg4,... ZEg14.
  # In the labels for the configurations, the first letter Y or Z
  # denotes the tree shape, with Y for symmetric and Z for asymmetric;
  # the second letter denotes the amount of ILS, with H for N=0.4 (`hard')
  # and E for N=4 (`easy'); and the rest denotes the number of loci,
  # either 4 or 14.
  # The numbers along the top are the number of times out of 50
  # that the clustering with the largest posterior probability
  # was not the true clustering.
  
  par(las=3, mar=c(5,5,1,1))
  boxplot(olave.Iss,
          names=cfg.names,
          boxwex=.5,
          ylab=expression("Index " ~ I[s]),
          col=rep(c("white","grey"),4))
  
  abline(v=6.5)
  olave.Is.means <- rep(0, length(cfg.names))
  for (cfg in 1:length(cfg.names)) {
    olave.Is.means[cfg] <- mean(olave.Iss[[cfg]])
  }
  points(x=1:length(cfg.names), y=olave.Is.means, pch=18, cex=1.5)
  
  # The boxplots show the measure of over-splitting of true species 
  # lineages using the index $I_s$ of Olave et al. The black
  # diamonds show the mean values. Note that the
  # vertical scale is about one tenth of Fig 3 in Olave et al.
  
  par(oldpar)
  }

do.olave.figure()

# J M Biol. For books and book-sized journals, the figures should be 80 mm or 122 mm wide
# and not higher than 198 mm (inc captions??).

pdf(file=article.pdf, width=80/25.4, height=120/25.4, pointsize=9)
do.olave.figure()
dev.off()

setEPS()
postscript(file=article.eps, width=80/25.4, height=120/25.4, pointsize=9)
do.olave.figure()
dev.off()

max(YHg4.join.misassign.score, YHg8.join.misassign.score, YHg14.join.misassign.score)
max(ZHg4.join.misassign.score, ZHg8.join.misassign.score, ZHg14.join.misassign.score)
max(YEg4.join.misassign.score, YEg8.join.misassign.score, YEg14.join.misassign.score)
max(ZEg4.join.misassign.score, ZEg8.join.misassign.score, ZEg14.join.misassign.score)


max(YHg4.max.false.pp, YHg8.max.false.pp, YHg14.max.false.pp)
max(ZHg4.max.false.pp, ZHg8.max.false.pp, ZHg14.max.false.pp)
max(YEg4.max.false.pp, YEg8.max.false.pp, YEg14.max.false.pp)
max(ZEg4.max.false.pp, ZEg8.max.false.pp, ZEg14.max.false.pp)


