#                             DISSECT.

# Started 2013-10-03. 
# This is for testing species delimitation algorithm DISSECT


library(ape)
library(phangorn)


olave.seqs.filepathbase <- function(repn, Ne, sptree.shape) {
  stopifnot(repn > 0)
  stopifnot(repn <= 50)
  stopifnot(identical(Ne, "4") || identical(Ne, "0.4"))
  stopifnot(identical(sptree.shape, as.integer(1)) || identical(sptree.shape, as.integer(2)))
  paste0("F:/AAA/Programming/ProgramOutput/Olave-data/replicate_", repn, "/", 
          Ne, "Ne_10Ind/", Ne, "Ne_10Ind_ST", sptree.shape, "_")
}

olave.nex.filepathbase <- function(repn, Ne, sptree.shape) {
  stopifnot(repn > 0)
  stopifnot(repn <= 50)
  stopifnot(identical(Ne, "4") || identical(Ne, "0.4"))
  fname.Ne <- Ne
  if (identical(fname.Ne, "0.4")) { fname.Ne <- "04" } 
  stopifnot(identical(sptree.shape, as.integer(1)) || identical(sptree.shape, as.integer(2)))
  paste0("F:/AAA/Programming/ProgramOutput/Olave-data/replicate_", repn, "/", 
          Ne, "Ne_10Ind/", fname.Ne, "Ne_10Ind_ST", sptree.shape, "_")
}

olave.taxa <- function() {
  taxa <- rep("",80)
  for (spp in 1:8) {
    for (ind in 1:5) {
      for (sq in 1:2) {
      taxa[10*(spp-1) + 2*(ind-1) + sq] <-
                 paste0(c("a","b","c","d","e","f","g","h")[spp],
                        c("01","02","03","04","05","06","07","08","09","10")[ind],
                        c("_A","_B")[sq])
        }
      }
    }
  taxa
}



olave.outdpath <- function() {
paste0(Args[1], "/")
}


olave.scenario <- function(Ne, sptree.shape, numgenes) {
stopifnot(numgenes > 0)
stopifnot(numgenes <= 14)
stopifnot(identical(Ne, "4") || identical(Ne, "0.4"))
stopifnot(identical(sptree.shape, as.integer(1)) || identical(sptree.shape, as.integer(2)))
scenario <- ""
if (sptree.shape == 1) {
  scenario <- "Z"
  } else {
  scenario <- "Y"
  }
if (identical(Ne, "4")) {
  scenario <- paste0(scenario, "E")
  } else {
  scenario <- paste0(scenario, "H")
  }
scenario <- paste0(scenario, "g", numgenes)
scenario
}



olave.scenario.outdpath <- function(Ne, sptree.shape, numgenes) {
paste0(olave.outdpath(), olave.scenario(Ne, sptree.shape, numgenes))
}



# Read in the sequences files and convert
convert.olave.data.to.nex <- function(nofgenes, taxa, seqs.fpathbase, nex.fpathbase) {
  for (g in 1:nofgenes) {
    numastext <- as.character(g)
    textlines <- readLines(paste0(seqs.fpathbase, numastext, "-Seq.phy"))
    sink(paste0(nex.fpathbase, numastext, "-Seq.nex"))
    cat("#NEXUS\n")
    cat("[Olave data converted to NEX]\n")
    cat("BEGIN DATA;\n")
    cat("\tDIMENSIONS NTAX=80 NCHAR=1000;\n")
    cat("\tFORMAT DATATYPE=DNA GAP=- MISSING=?;\n")
    cat("\tMATRIX\n")
    numastext <- as.character(g)
    textlines <- readLines(paste0(seqs.fpathbase, numastext, "-Seq.phy"))
    n <- length(textlines)
    tokens <- strsplit(textlines, split=" +")
    for (i in 2:n) {
      wantedtaxon <- taxa[i-1]
      seqnce <- NULL
      for (j in 2:n) {
        taxon <- taxa[ as.integer(tokens[[j]][1]) ]
        if (identical(taxon, wantedtaxon)) {
          stopifnot(is.null(seqnce)) 
          seqnce <- tokens[[j]][2]
          }
        }
      stopifnot(!is.null(seqnce))  
      taxon <- taxa[ as.integer(tokens[[i]][1]) ]
      seqnce <- tokens[[i]][2]
      cat("\t", wantedtaxon, "     ", seqnce, "\n\n")
      }
    cat(";\n")
    cat("END;\n")
    sink(NULL)
    }
  }





################################################################################
########################## Business starts here ################################


cat("Converting olave data to nex", "\n")
 
Ne <- "N04"
sptree.shape <- "Y"
numgenes <- "g14"
rep.nums <- "r01-50"

 
stopifnot(identical(Ne, "N4") || identical(Ne, "N04"))
if (Ne == "N4") { Ne <- "4" } else { Ne <- "0.4" }

stopifnot(identical(sptree.shape, "Z") || identical(sptree.shape,"Y"))
if (sptree.shape == "Z") { sptree.shape <- as.integer(1) } else { sptree.shape <- as.integer(2) }

stopifnot(substr(numgenes, 1, 1) == "g")
substr(numgenes, 2, nchar(numgenes))
numgenes <- as.integer(substr(numgenes, 2, nchar(numgenes)))
numgenes
stopifnot(is.finite(numgenes))
stopifnot(numgenes > 0)
stopifnot(numgenes <= 14)

stopifnot(nchar(rep.nums) == 6)
stopifnot(substr(rep.nums, 1, 1) == "r")
stopifnot(substr(rep.nums, 4, 4) == "-")
first.rep.num <- as.integer(substr(rep.nums, 2, 3))
last.rep.num <- as.integer(substr(rep.nums, 5, 6))
stopifnot(first.rep.num  > 0)
stopifnot(first.rep.num  <= last.rep.num)
stopifnot(last.rep.num  <= 50)


Ne
sptree.shape
numgenes
first.rep.num
last.rep.num


for (r in first.rep.num:last.rep.num) {
  seqs.fpathbase <- olave.seqs.filepathbase(r, Ne, sptree.shape)
  nex.filepathbase <- olave.nex.filepathbase(r, Ne, sptree.shape)
  convert.olave.data.to.nex(numgenes, olave.taxa(), seqs.fpathbase, nex.filepathbase)
  }


