plot.rectangle <- function(v1,v2,...)
{
polygon(c(v1[1],v2[1],v2[1],v1[1]), c(v1[2],v1[2],v2[2],v2[2]), ...)
}

# Main plotting routine
plot.simmatrix <- function(sim, nmincls, mincl.names) {
  par(mar= c(0,5,5,0)+.1)
  plot(NULL, xlim=c(0,nmincls), ylim=c(0,nmincls), axes=FALSE, ylab="", xlab="")
  axis(3, at=(1:nmincls)-.5, mincl.names, tick=FALSE, las=2, line=-1)
  axis(2, at=(1:nmincls)-.5, rev(mincl.names), tick=FALSE, las=2, line=-1)
  for (i in 1:nmincls) {
    for (j in 1:nmincls) {
      d <- 1 - sim[i,j]
      plot.rectangle(c(i-1,nmincls-j), c(i,nmincls-j+1), col=rgb(d,d,d), border="white")
    }
  }
#  for (b in dividers) {
#    lines(x=c(-.5,nmincls), y=c(nmincls-b,nmincls-b))
#    lines(x=c(b,b), y=c(0,nmincls+.5))
#  }                    
}


make.simmatrix <- function(clusterings.fpath, simmatrix.fpath, simmatrix.pdf.fpath) {
  # Read in the table of clusterings
  x <- read.table(clusterings.fpath, header=TRUE)
  # Minimal cluster names are column names omitting first 4 
  mincl.names <- colnames(x)[-(1:4)]
  # Make the similarity matrix  
  nmincls <- length(mincl.names)
  sim <- matrix(0, ncol=nmincls, nrow=nmincls, dimnames=list(mincl.names, mincl.names))
  for (i in 1:nmincls) {
    for (j in 1:nmincls) {
      coli <- x[,mincl.names[i]]
      colj <- x[,mincl.names[j]]
      w <- coli == colj
      sim[i,j] <- sum(x[w,"fraction"])
    }
  }
  # Display as text, on screen, and to PDF file
  print(sim, digits=2)
  #plot.simmatrix(sim, nmincls, mincl.names)
  pdf(file=simmatrix.pdf.fpath)
  plot.simmatrix(sim, nmincls, mincl.names)
  dev.off()  
}





