#                             STACEY

# Started 2014-09-11. 
# This is for testing species delimitation in STACEY on Olave data


#   Copyright Graham Jones 2014

#   This code is released under the Creative Commons 
#   Attribution-NonCommercial 4.0 International (CC BY-NC 4.0)
#   licence, found here:
#   http://creativecommons.org/licenses/by-nc/4.0/

 




library(ape)
library(phangorn)

################################################################################
###################### Setting up file paths, loading code #####################

cat("STACEY ('Olave data' version) started\n") 

Args <- commandArgs(TRUE)

# Comment the next line out when running from outside R
# Args <- c("C:/Users/Work/AAA/Programming/ProgramOutput/Olave-STACEY", "N4", "Y", "g1", "r02-02")   



stopifnot(length(Args) == 5) 

# Where this code is
setwd("C:/Users/Work/AAA/Programming/biology/STACEYOlaveSimulations")

# BEAST2 jar
BEAST.jar.fpath <- "C:\\Users\\Work\\AAA\\biology\\SoftWare\\BEAST.v2.2.0\\lib\\beast.jar"

# Should just be able to do "java" but doesn't work on my system
# when .BAT file calls R calls java
java.fpath <- "\"C:\\Program Files\\Java\\jre1.8.0_31\\bin\\java.exe\""

# where the data is
Olave.data.filepath.prefix <- "C:/Users/Work/AAA/Programming/ProgramOutput/Olave-data/replicate_"


# Code providing functions
source("STACEY_5beast2xml_toplevel.r")
source("STACEY_6beast2xml_utils.r")
source("STACEY_6beast2xml_ids.r")
source("STACEY_6beast2xml_data.r")
source("STACEY_6beast2xml_distribution.r")
source("STACEY_6beast2xml_init.r")
source("STACEY_6beast2xml_state.r")
source("STACEY_6beast2xml_operators.r")
source("STACEY_6beast2xml_loggers.r")
source("STACEY_7simmatrix.r")




################################################################################
######################### Top level functions ##################################


olave.seqs.filepathbase <- function(repn, Ne, sptree.shape, numgenes) {
  stopifnot(repn > 0)
  stopifnot(repn <= 50)
  stopifnot(numgenes > 0)
  stopifnot(numgenes <= 14)
  stopifnot(identical(Ne, "4") || identical(Ne, "0.4"))
  stopifnot(identical(sptree.shape, as.integer(1)) || identical(sptree.shape, as.integer(2)))
  paste0(Olave.data.filepath.prefix, repn, "/", 
          Ne, "Ne_10Ind/", Ne, "Ne_10Ind_ST", sptree.shape, "_")
}


olave.taxa <- function() {
  taxa <- rep("",80)
  for (spp in 1:8) {
    for (ind in 1:5) {
      for (sq in 1:2) {
      taxa[10*(spp-1) + 2*(ind-1) + sq] <-
                 paste0(c("a","b","c","d","e","f","g","h")[spp],
                        c("01","02","03","04","05")[ind],
                        c("A","B")[sq])
        }
      }
    }
  taxa
}




olave.beast.priors <- function() {  
list(popmean=list(type="lnorm", par1="-7", par2="2"),
     growthrate=list(type="lnorm", par1="4.6", par2="2"),
     reldeathrate=list(type="beta", par1="3", par2="1"),
     relativerates=list(type="lnorm", par1="0", par2="1"),
     collapseweight=list(type="beta", par1="1", par2="1"))
}



olave.outdpath <- function() {
paste0(Args[1], "/")
}


olave.scenario <- function(Ne, sptree.shape, numgenes) {
stopifnot(numgenes > 0)
stopifnot(numgenes <= 14)
stopifnot(identical(Ne, "4") || identical(Ne, "0.4"))
stopifnot(identical(sptree.shape, as.integer(1)) || identical(sptree.shape, as.integer(2)))
scenario <- ""
if (sptree.shape == 1) {
  scenario <- "Z"
  } else {
  scenario <- "Y"
  }
if (identical(Ne, "4")) {
  scenario <- paste0(scenario, "E")
  } else {
  scenario <- paste0(scenario, "H")
  }
scenario <- paste0(scenario, "g", numgenes)
scenario
}

olave.scenario.outdpath <- function(Ne, sptree.shape, numgenes) {
paste0(olave.outdpath(), olave.scenario(Ne, sptree.shape, numgenes))
}


olave.fpath.dbug <- function(r, Ne, sptree.shape, numgenes) {
  paste0(olave.outdpath(), olave.scenario(Ne, sptree.shape, numgenes), "/", "9DBUG", "r", r, ".txt")  
}

olave.fpath.sampledgtrees.fpathbase <- function(r, Ne, sptree.shape, numgenes) {
paste0(olave.outdpath(), olave.scenario(Ne, sptree.shape, numgenes), "/", "4GTS", "r", r, "-")
}


olave.fpath.sampledspptrees <- function(r, Ne, sptree.shape, numgenes) {
paste0(olave.outdpath(), olave.scenario(Ne, sptree.shape, numgenes), "/", "4STS", "r", r, ".txt")
} 


olave.fpath.sampledparams <- function(r, Ne, sptree.shape, numgenes) {
paste0(olave.outdpath(), olave.scenario(Ne, sptree.shape, numgenes), "/", "4LOG", "r", r, ".txt")
} 


olave.beastxml.fpath <- function(r, Ne, sptree.shape, numgenes) {
paste0(olave.outdpath(), olave.scenario(Ne, sptree.shape, numgenes), "/", "3BX", "r", r, ".xml")
}


olave.beast.state.fpath <- function(r, Ne, sptree.shape, numgenes) {
  paste0(olave.outdpath(), olave.scenario(Ne, sptree.shape, numgenes), "/", "3BX", "r", r, "_STATE.xml")
}



olave.fpath.mccgtree <- function(r, Ne, sptree.shape, numgenes, g) {
paste0(olave.outdpath(), olave.scenario(Ne, sptree.shape, numgenes), "/", "5GTC", "r", r, "-", g, ".txt")
}

 
olave.fpath.mccspptree <- function(r, Ne, sptree.shape, numgenes)   {
paste0(olave.outdpath(), olave.scenario(Ne, sptree.shape, numgenes), "/", "5STC", "r", r, ".txt")
}


olave.fpath.clusterings <- function(r, Ne, sptree.shape, numgenes) {
paste0(olave.outdpath(), olave.scenario(Ne, sptree.shape, numgenes), "/", "6CLU", "r", r, ".txt")
}  

olave.fpath.simmatrix <- function(r, Ne, sptree.shape, numgenes) {
paste0(olave.outdpath(), olave.scenario(Ne, sptree.shape, numgenes), "/", "7SMM", "r", r, ".txt")
}  

olave.fpath.simmatrix.pdf <- function(r, Ne, sptree.shape, numgenes) {
paste0(olave.outdpath(), olave.scenario(Ne, sptree.shape, numgenes), "/", "7SMM", "r", r, ".pdf")
}  



make.beastxml.for.olave.one.replicate <- function(r, Ne, sptree.shape, numgenes) {  

  sim.options <- list(msc.model="STACEYD",
                      doing.olave=TRUE,
                      nofgenes=numgenes,
                      taxa=olave.taxa(),
                      beast.priors=olave.beast.priors(),
                      beast.collapseheight=BEASTXML.collapseheight,
                      seqs.fpathbase=olave.seqs.filepathbase(r, Ne, sptree.shape, numgenes), 
                      sampledgtrees.fpathbase=olave.fpath.sampledgtrees.fpathbase(r, Ne, sptree.shape, numgenes),
                      sampledspptrees.fpath=olave.fpath.sampledspptrees(r, Ne, sptree.shape, numgenes),
                      sampledparams.fpath=olave.fpath.sampledparams(r, Ne, sptree.shape, numgenes),
                      DBUGTUNE.fpath=olave.fpath.dbug(r, Ne, sptree.shape, numgenes),
                      beastchainlength=BEASTXML.chain.length,
                      beastparams.logevery=BEASTXML.params.logevery,
                      beastspptree.logevery=BEASTXML.spptree.logevery,
                      beastgtrees.logevery=BEASTXML.gtrees.logevery,
                      beastscreen.logevery=BEASTXML.screen.logevery                       
  )
  
  sink(olave.beastxml.fpath(r, Ne, sptree.shape, numgenes))
  make.beast2xml.from.sim.data(sim.options)
  sink(NULL)
  }


beast.cmd.for.olave.one.replicate <- function(r, Ne, sptree.shape, numgenes) {
  paste0(java.fpath, " -enableassertions -jar ", BEAST.jar.fpath, " ",
        "-overwrite ", 
        "-statefile ", olave.beast.state.fpath(r, Ne, sptree.shape, numgenes), " ",
        olave.beastxml.fpath(r, Ne, sptree.shape, numgenes))
}




run.beast.for.olave.one.replicate <- function(r, Ne, sptree.shape, numgenes) {
  BEAST.cmd <- beast.cmd.for.olave.one.replicate(r, Ne, sptree.shape, numgenes)
  cat(BEAST.cmd, "\n")
  errorcode <- system(BEAST.cmd)
  stopifnot(errorcode == 0)  # don't seem to work
  }
  



treeannotator.cmds.for.olave.one.replicate <- function(r, Ne, sptree.shape, numgenes, treeann) {
  TreeAnnotator.cp <-
    paste0(java.fpath, " -classpath ",
           getwd(), "/", "beast.jar ",
           "dr.app.tools.TreeAnnotator ")
  cmds <- rep("", numgenes)
  for (g in 1:numgenes) {
    cmds[g] <- 
      paste0(TreeAnnotator.cp,
             "-burnin ", treeann$gtrees.burnin, " ",
             olave.fpath.sampledgtrees.fpathbase(r, Ne, sptree.shape, numgenes), g, ".txt ",
             olave.fpath.mccgtree(r, Ne, sptree.shape, numgenes, g))
    
  } 
  cmds[numgenes + 1] <- paste0(TreeAnnotator.cp,
                               "-burnin ", treeann$stree.burnin, " ",
                               olave.fpath.sampledspptrees(r, Ne, sptree.shape, numgenes), " ",
                               olave.fpath.mccspptree(r, Ne, sptree.shape, numgenes))
  cmds
}




run.treeannotator.for.olave.one.replicate <- function(r, Ne, sptree.shape, numgenes, treeann) {
  TreeAnnotator.cmds <- treeannotator.cmds.for.olave.one.replicate(r, Ne, sptree.shape, numgenes, treeann)
  for (c in 1:length(TreeAnnotator.cmds)) {
    system(TreeAnnotator.cmds[c])
  }
}  


speciesdelimanalyser.cmd.for.olave.one.replicate <- function(r, Ne, sptree.shape, numgenes, sppdelimanal) {
  paste0(java.fpath, " -jar ", getwd(), "/", "speciesDA.jar ",
         "-burnin ", sppdelimanal$burnin, " ",
         "-collapseheight ", sppdelimanal$collapseheight, " ",
         "-simcutoff ", sppdelimanal$simcutoff, " ",
         olave.fpath.sampledspptrees(r, Ne, sptree.shape, numgenes), " ",
         olave.fpath.clusterings(r, Ne, sptree.shape, numgenes))  
}


run.speciesdelimanalyser.for.olave.one.replicate <- function(r, Ne, sptree.shape, numgenes, sppdelimanal) {
  SpeciesDelimitationAnalyser.cmd <- speciesdelimanalyser.cmd.for.olave.one.replicate(r, Ne, sptree.shape, numgenes, sppdelimanal)
  system(SpeciesDelimitationAnalyser.cmd)
}



make.simmatrix.for.olave.one.replicate <- function(r, Ne, sptree.shape, numgenes) {
  make.simmatrix(olave.fpath.clusterings(r, Ne, sptree.shape, numgenes),
                 olave.fpath.simmatrix(r, Ne, sptree.shape, numgenes),
                 olave.fpath.simmatrix.pdf(r, Ne, sptree.shape, numgenes)) 
} 



################################################################################
########################## Business starts here ################################


cat("STACEY (with delimitation) running with olave data", "\n")

BEASTXML.collapseheight <- .0001
BEASTXML.chain.length <- "7000000"
BEASTXML.screen.logevery <- 10000
BEASTXML.params.logevery <- 1000
BEASTXML.gtrees.logevery <- 20000
BEASTXML.spptree.logevery <- 1000
BEASTXML.dbugtune.logevery <- 10000

gtrees.burnin <- 101
common.burnin <- 2001

treeann <- list(gtrees.burnin=gtrees.burnin, stree.burnin=common.burnin)
sppdelimanal <- list(burnin=1001, collapseheight=BEASTXML.collapseheight, simcutoff=1.0) 
 
Ne <- Args[2]
sptree.shape <- Args[3]
numgenes <- Args[4]
rep.nums <- Args[5]
 
stopifnot(identical(Ne, "N4") || identical(Ne, "N04"))
if (Ne == "N4") { Ne <- "4" } else { Ne <- "0.4" }

stopifnot(identical(sptree.shape, "Z") || identical(sptree.shape,"Y"))
if (sptree.shape == "Z") { sptree.shape <- as.integer(1) } else { sptree.shape <- as.integer(2) }

stopifnot(substr(numgenes, 1, 1) == "g")
numgenes <- as.integer(substr(numgenes, 2, nchar(numgenes)))
stopifnot(is.finite(numgenes))
stopifnot(numgenes > 0)
stopifnot(numgenes <= 14)

stopifnot(nchar(rep.nums) == 6)
stopifnot(substr(rep.nums, 1, 1) == "r")
stopifnot(substr(rep.nums, 4, 4) == "-")
first.rep.num <- as.integer(substr(rep.nums, 2, 3))
last.rep.num <- as.integer(substr(rep.nums, 5, 6))
stopifnot(first.rep.num  > 0)
stopifnot(first.rep.num  <= last.rep.num)
stopifnot(last.rep.num  <= 50)

cat("Ne = ", Ne, "\n")
cat("sptree.shape = ", sptree.shape, "\n")
cat("numgenes = ", numgenes, "\n")
cat("first.rep.num = ", first.rep.num, " ")
cat("last.rep.num = ", last.rep.num, "\n")


if (!file.exists(olave.scenario.outdpath(Ne, sptree.shape, numgenes))) {
  dir.create(olave.scenario.outdpath(Ne, sptree.shape, numgenes))
}
 
for (r in first.rep.num:last.rep.num) {
  make.beastxml.for.olave.one.replicate(r, Ne, sptree.shape, numgenes)
  }

for (r in first.rep.num:last.rep.num) {
  run.beast.for.olave.one.replicate(r, Ne, sptree.shape, numgenes)
  }


# for (r in first.rep.num:last.rep.num) {
#   run.treeannotator.for.olave.one.replicate(r, Ne, sptree.shape, numgenes, treeann)
# }


for (r in first.rep.num:last.rep.num) {
  run.speciesdelimanalyser.for.olave.one.replicate(r, Ne, sptree.shape, numgenes, sppdelimanal)
}


for (r in first.rep.num:last.rep.num) {
  make.simmatrix.for.olave.one.replicate(r, Ne, sptree.shape, numgenes)
}



