# This version is for a set of G,N,T values,
# (G = nof genes, N=nof indivs, T = mutation rate)
# OR a set of GT values and indivs supplied in tree
# (but not both)


##################################################################################

all.files.prefix.from.config.dir <- function(config.dir) {
  n <- nchar(config.dir)
  last <- substr(config.dir, n, n)
  stopifnot(grepl("[ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz]", last))
  last
}
              
              
               
# file name components - 'type'
fnc.spptree <- "1ST"
fnc.simgtree <- "2GT"
fnc.simseq <- "2SQ"
fnc.upgmagtrees <- "2UGT"
fnc.beastxml <- "3BX"
fnc.sampledgtrees <- "4GTS"
fnc.sampledspptrees <- "4STS"
fnc.sampledparams <- "4LOG"
fnc.concensusgtree <- "5GTC"
fnc.concensusspptree <- "5STC"
fnc.clusterings <- "6CLU"
fnc.simmatrix <- "7SMM"
fnc.DBUGTUNE <- "9DBUGTUNE"
# file name components which appear before config params
fnc.nofgenes <- "g"
fnc.nofindivs <- "n"
fnc.mutrate <- "t"
fnc.replicate <- "r"



fpath.GNTname <- function(config, i) {
  G <- config$configG[i]
  N <- config$configN[i]
  mu <- config$configT[i]
  R <- config$configR[i]
  if (N > 0) {
    GNTname <- paste(fnc.nofgenes, G, fnc.nofindivs, N, 
                   fnc.mutrate, as.integer(round(mu*1e9)), sep="")
    } else {
    GNTname <- paste(fnc.nofgenes, G, 
                   fnc.mutrate, as.integer(round(mu*1e9)), sep="")
    }
  GNTname                 
  }
  
  
fpath.GNTRname <- function(config, i, r) {
  GNTRname <- paste(fpath.GNTname(config, i), fnc.replicate, r, sep="")
  GNTRname
  }


fpath.GNTdpath <- function(config, i) {
  GNTdpath <- paste(config$dpath, "/", config$a.f.p, fpath.GNTname(config, i), sep="")
  GNTdpath
  }
  
fpath.seed.name <- function(config) {
  seed <- config$progparams$beastseed
  if (seed > 0) {
    sotext <- paste0("s", seed)
  } else {
    sotext <- ""
  }
  
  sotext
}

  
fpath.spptree <- function(config, i) {
  spptree.fpath <- paste(fpath.GNTdpath(config, i), "/", config$a.f.p, 
                   fnc.spptree, fpath.GNTname(config, i), ".txt", sep="")
  spptree.fpath
  }

fpath.spptree.pdf <- function(config, i) {
  spptree.fpath.pdf <- paste(fpath.GNTdpath(config, i), "/", config$a.f.p, 
                   fnc.spptree, fpath.GNTname(config, i), ".pdf", sep="")
  spptree.fpath.pdf
  }


       
       
fpath.gtrees.fpathbase <- function(config, i, r) {
  gtrees.fpathbase <- paste(fpath.GNTdpath(config, i), "/", config$a.f.p, 
        fnc.simgtree, fpath.GNTRname(config, i, r), "-", sep="")
  gtrees.fpathbase
  }
  
fpath.gtree <- function(config, i, r, g) {
  gtree.fpath <- paste(fpath.gtrees.fpathbase(config, i, r), g, ".txt", sep="")
  gtree.fpath
  }
  
fpath.gtree.pdf <- function(config, i, r, g) {
  gtree.fpath <- paste(fpath.gtrees.fpathbase(config, i, r), g, ".pdf", sep="")
  gtree.fpath
  }

  
fpath.seqs.fpathbase <- function(config, i, r) {
  seqs.fpathbase <- paste(fpath.GNTdpath(config, i), "/", config$a.f.p, 
        fnc.simseq, fpath.GNTRname(config, i, r), "-", sep="")
  seqs.fpathbase
  }
  
fpath.seq <- function(config, i, r, g) {
  seq.fpath <- paste(fpath.seqs.fpathbase(config, i, r), g, ".txt", sep="")
  seq.fpath
  }

fpath.seq.nex <- function(config, i, r, g) {
  nex.fpath <- paste(fpath.seqs.fpathbase(config, i, r), g, ".nex", sep="")
  nex.fpath
}


fpath.upgmagtrees.fpathbase <- function(config, i, r) {
  upgmagtrees.fpathbase <- paste(fpath.GNTdpath(config, i), "/", config$a.f.p, 
        fnc.upgmagtrees, fpath.GNTRname(config, i, r), "-", sep="")
  upgmagtrees.fpathbase
  }
  
fpath.upgmagtree.pdf <- function(config, i, r, g) {
  upgmagtree.pdf.fpath <- paste(fpath.upgmagtrees.fpathbase(config, i, r), g, ".pdf", sep="")
  upgmagtree.pdf.fpath
  } 



fpath.beastxml <- function(config, i, r, operators.choice) {
  beastxml <- paste0(fpath.GNTdpath(config, i), "/", 
                     config$a.f.p, 
                     fnc.beastxml,
                     fpath.GNTRname(config, i, r), 
                     fpath.seed.name(config), 
                     operators.choice,
                     ".xml")
  beastxml
  }

fpath.sampledgtrees.fpathbase <- function(config, i, r, operators.choice) {
  sampledgtrees.fpathbase <- paste0(config$a.f.p, 
                                    fnc.sampledgtrees,
                                    fpath.GNTRname(config, i, r), 
                                    fpath.seed.name(config),
                                    operators.choice,
                                    "-")
  sampledgtrees.fpathbase
  }


  
fpath.sampledspptrees <- function(config, i, r, operators.choice) {
  sampledspptrees.fpath <- paste0(config$a.f.p,
                                  fnc.sampledspptrees, 
                                  fpath.GNTRname(config, i, r),
                                  fpath.seed.name(config),
                                  operators.choice,
                                  ".txt")
  sampledspptrees.fpath
  }

  
fpath.sampledparams <- function(config, i, r, operators.choice) {  
  sampledparams.fpath <- paste0(config$a.f.p,
                                fnc.sampledparams, 
                                fpath.GNTRname(config, i, r),
                                fpath.seed.name(config),
                                operators.choice,
                                ".txt")
  sampledparams.fpath
  }

fpath.DBUGTUNE <- function(config, i, r) {
  DBUGTUNE.fpath <- paste(fpath.GNTdpath(config, i), "/", config$a.f.p, 
                   fnc.DBUGTUNE, fpath.GNTRname(config, i, r), ".txt", sep="")
  DBUGTUNE.fpath
  }
  

fpath.mccgtrees.fpathbase <- function(config, i, r) {
  mccgtrees.fpathbase <- paste(fpath.GNTdpath(config, i), "/", config$a.f.p, 
           fnc.concensusgtree, fpath.GNTRname(config, i, r), "-", sep="")
  mccgtrees.fpathbase
  }
  
  
fpath.mccgtree <- function(config, i, r, g) {
  mccgtree.fpath <- paste(fpath.mccgtrees.fpathbase(config, i, r), g, ".txt", sep="")
  mccgtree.fpath
  }
  
fpath.mccgtree.pdf <- function(config, i, r, g) {
  mccgtree.fpath <- paste(fpath.mccgtrees.fpathbase(config, i, r), g, ".pdf", sep="")
  mccgtree.fpath
  }  

fpath.mccspptree <- function(config, i, r) {
  mccspptree.fpath <- paste(fpath.GNTdpath(config, i), "/", config$a.f.p, 
           fnc.concensusspptree, fpath.GNTRname(config, i, r), ".txt", sep="")
  mccspptree.fpath
  }
  
fpath.mccspptree.pdf <- function(config, i, r) {
  mccspptree.fpath <- paste(fpath.GNTdpath(config, i), "/", config$a.f.p, 
           fnc.concensusspptree, fpath.GNTRname(config, i, r), ".pdf", sep="")
  mccspptree.fpath
  }
  
  
fpath.clusterings <- function(config, i, r) {
  clusterings.fpath <- paste(fpath.GNTdpath(config, i), "/", config$a.f.p, 
           fnc.clusterings, fpath.GNTRname(config, i, r), ".txt", sep="")
  clusterings.fpath
  }  

fpath.simmatrix <- function(config, i, r) {
  simmatrix.fpath <- paste(fpath.GNTdpath(config, i), "/", config$a.f.p, 
           fnc.simmatrix, fpath.GNTRname(config, i, r), ".txt", sep="")
  simmatrix.fpath
  } 

fpath.simmatrix.pdf <- function(config, i, r) {
  simmatrix.fpath.pdf <- paste(fpath.GNTdpath(config, i), "/", config$a.f.p, 
           fnc.simmatrix, fpath.GNTRname(config, i, r), ".pdf", sep="")
  simmatrix.fpath.pdf
  } 

#################################################################################


# For version that uses T and pops, not theta, 
# and either same number of indivs for all species and all GNTs
# or allows differing indivs per species, but all GNTs get these
read.config <- function() {
  filepath <- paste(CONFIG.DIR, "/", "config.txt", sep="")
  a.f.p <- all.files.prefix.from.config.dir(CONFIG.DIR)
  all.lines <- readLines(filepath)
  ppline <- which(grepl("PROGRAMPARAMETERS", all.lines))
  if (length(ppline) == 0) {
    progparams <- list(beastseed=42, beastchainlength=1500000,
           beastscreen.logevery=10000, beastparams.logevery=1000,
           beastgtrees.logevery=1000, beastspptree.logevery=1000,
           beastdbugtune.logevery=10000, treeannburnin=501)
    config.lines <- all.lines
    } else {
    stopifnot(length(ppline) == 1)
    scenline <- which(grepl("SCENARIO", all.lines)) 
    stopifnot(length(scenline) == 1)
    config.lines <- all.lines[ (scenline+1) : length(all.lines) ]
    progparam.lines <- all.lines[ (ppline+1) : (scenline-1) ]
    progparams <- parse.progparams(progparam.lines)
    }

  tokens <- strsplit(config.lines, split="[[:blank:]]+")
  
  linenum <- 1
  stopifnot(tokens[[linenum]][1] == "nofGNRT")
  stopifnot(length(tokens[[linenum]]) == 2)
  nofGNRT <- as.integer(tokens[[linenum]][2])
  
  nofconfigs <- 0
  ntokens.before.Ts <- 4
  for (i in 1:nofGNRT) {
    ntokens <- length(tokens[[linenum+i]])
    stopifnot(ntokens > ntokens.before.Ts)
    nofconfigs <- nofconfigs + ntokens - ntokens.before.Ts
    }
  
  configG <- rep(0, nofconfigs)
  configN <- rep(0, nofconfigs)
  configR0 <- rep(0, nofconfigs)
  configR1 <- rep(0, nofconfigs)
  configT <- rep(0, nofconfigs)
  cfg <- 1
  for (i in 1:nofGNRT) {
    Gtoken <- tokens[[1+i]][1]
    Ntoken <- tokens[[1+i]][2]
    R0token <- tokens[[1+i]][3]
    R1token <- tokens[[1+i]][4]
    G <- as.integer(Gtoken)
    if (Ntoken == "*") {
      N <- -1
      } else { 
      N <- as.integer(Ntoken)
      }
    R0 <- as.integer(R0token)
    R1 <- as.integer(R1token)
    nofTs <- length(tokens[[1+i]]) - ntokens.before.Ts
    for (j in 1:nofTs) {
      configG[cfg] <- G
      configN[cfg] <- N
      configR0[cfg] <- R0
      configR1[cfg] <- R1
      configT[cfg] <- as.double(tokens[[1+i]][ntokens.before.Ts+j])
      cfg <- cfg + 1
      }
    }
  
  linenum <- nofGNRT + 2
  stopifnot(tokens[[linenum]][1] == "genelength")
  stopifnot(length(tokens[[linenum]]) == 2)
  genelength <- as.integer(tokens[[linenum]][2])
  
  
  linenum <- nofGNRT + 3
  stopifnot(tokens[[linenum]][1] == "nofspp")
  stopifnot(length(tokens[[linenum]]) == 2)
  nofspp <- as.integer(tokens[[linenum]][2])

  if (genelength == 0) {
    stopifnot(identical(nofconfigs, 1))
    stopifnot(identical(configG[1], 1))
    stopifnot(identical(configN[1], 1))
    stopifnot(identical(configR[1], 1))
    sppnodes <- NULL
  } else {
    sppnodes <- read.spptree.nodes(tokens, linenum, nofspp)
    check.sppnodes(sppnodes, nofspp)
  }
  spptree <- list(nofspp=nofspp, sppnodes=sppnodes)                        
  seqs <- list(len=genelength)
  list(progparams=progparams, configG=configG, configN=configN, configT=configT,
         configR0=configR0, configR1=configR1, 
         seqs=seqs, spptree=spptree, dpath=CONFIG.DIR, a.f.p=a.f.p)
  }
  

check.sppnodes <- function(sppnodes, nofspp) {
  stopifnot(2*nofspp - 1 == length(sppnodes))
  }

  
  
read.spptree.nodes <- function(tokens, startlinenum, nofspp) {
  nofsppnodes <- 2*nofspp - 1
  linenum  <- startlinenum
  nodes <- as.list(NULL)
  for (i in 1:nofsppnodes) {
    linenum <- startlinenum + i
    node <- read.sppnode(tokens, linenum, (i <= nofspp), (i == nofsppnodes))
    nodes <- c(nodes, list(node))
    }
  nodes
  }
  

  
  
  
  
read.sppnode <- function(tokens, linenum, tip, root) {
  stopifnot(length(tokens[[linenum]]) == 9)
  nheight <- 0
  tippop <- -1
  rootpop <- -1 
  nlins <- -1  
  clade <- tokens[[linenum]][1]
  
  if (tip) {
    stopifnot(nchar(clade) == 1)
    } else {
    stopifnot(nchar(clade) >= 5)
    }
  
  stopifnot(tokens[[linenum]][2] == "nindivs")
  if (tip) {
    if (tokens[[linenum]][3] != "*") {
      nlins <- as.double(tokens[[linenum]][3]) 
      stopifnot(is.finite(nlins))
      }
    } 
  
  stopifnot(tokens[[linenum]][4] == "hgt")
  nheight <- as.double(tokens[[linenum]][5])
  stopifnot(is.finite(nheight))
  if (tip) {
    stopifnot(nheight == 0)
    } else {
    stopifnot(nheight > 0)
    }
    
  stopifnot(tokens[[linenum]][6] == "tipp")
  if (tip) {
    tippop <- as.double(tokens[[linenum]][7])
    stopifnot(is.finite(tippop))
    }
  
  stopifnot(tokens[[linenum]][8] == "rootp")
  if (tokens[[linenum]][9] != "*") {
    rootpop <- as.double(tokens[[linenum]][9])
    stopifnot(is.finite(rootpop))
    } else {
    stopifnot(root)
    }
    
  list(clade=clade, nheight=nheight,
       tippop=tippop, rootpop=rootpop,
       root=root, nlins=nlins)  
  } 





