
parse.prior <- function(tokens) {
  type <- tokens[1]
  stopifnot(  identical(type, "exp")  ||  
                identical(type, "beta")  ||  
                identical(type, "fixed")  ||  
                identical(type, "lnorm"))
  par1 <- as.double(tokens[2])
  par2 <- as.double(tokens[3])
  stopifnot(is.finite(par1))
  stopifnot(is.finite(par2))
  list(type=type, par1=par1, par2=par2)
}


parse.progparams <- function(progparam.lines) {
  tokens <- strsplit(progparam.lines, split="[[:blank:]]+")
  
  linenum <- 1
  stopifnot(tokens[[linenum]][1] == "beast.popmean.prior")
  stopifnot(length(tokens[[linenum]]) == 4)
  beast.popmean.prior <- parse.prior(tokens[[linenum]][2:4])
  
  linenum <- 2
  stopifnot(tokens[[linenum]][1] == "beast.growthrate.prior")
  stopifnot(length(tokens[[linenum]]) == 4)
  beast.growthrate.prior <- parse.prior(tokens[[linenum]][2:4])
  
  linenum <- 3
  stopifnot(tokens[[linenum]][1] == "beast.reldeathrate.prior")
  stopifnot(length(tokens[[linenum]]) == 4)
  beast.reldeathrate.prior <- parse.prior(tokens[[linenum]][2:4])
  
  linenum <- 4
  stopifnot(tokens[[linenum]][1] == "beast.relativerates.prior")
  stopifnot(length(tokens[[linenum]]) == 4)
  beast.relativerates.prior <- parse.prior(tokens[[linenum]][2:4]) 
  
  linenum <- 5
  stopifnot(tokens[[linenum]][1] == "beast.collapseweight.prior")
  stopifnot(length(tokens[[linenum]]) == 4)
  beast.collapseweight.prior <- parse.prior(tokens[[linenum]][2:4])
  
  beast.priors <- list(popmean=beast.popmean.prior,
                       growthrate=beast.growthrate.prior,
                       reldeathrate=beast.reldeathrate.prior,
                       relativerates=beast.relativerates.prior,
                       collapseweight=beast.collapseweight.prior,
                       estimate.popmean = !identical(beast.popmean.prior$type, "fixed"),
                       estimate.growthrate = !identical(beast.growthrate.prior$type, "fixed"),
                       estimate.reldeathrate = !identical(beast.reldeathrate.prior$type, "fixed"),
                       estimate.relativerates = !identical(beast.relativerates.prior$type, "fixed"),
                       estimate.collapseweight = !identical(beast.collapseweight.prior$type, "fixed")
                       )                     
                       
  linenum <- 6
  stopifnot(tokens[[linenum]][1] == "beast.collapseheight")
  stopifnot(length(tokens[[linenum]]) == 2)
  beast.collapseheight <- as.double(tokens[[linenum]][2])                       
  
  linenum <- 7
  stopifnot(tokens[[linenum]][1] == "beastseed")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastseed <- as.integer(tokens[[linenum]][2])

  linenum <- 8
  stopifnot(tokens[[linenum]][1] == "beastchainlength")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastchainlength <- as.integer(tokens[[linenum]][2])

  linenum <- 9
  stopifnot(tokens[[linenum]][1] == "beastscreen.logevery")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastscreen.logevery <- as.integer(tokens[[linenum]][2])

  linenum <- 10
  stopifnot(tokens[[linenum]][1] == "beastparams.logevery")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastparams.logevery <- as.integer(tokens[[linenum]][2])

  linenum <- 11
  stopifnot(tokens[[linenum]][1] == "beastgtrees.logevery")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastgtrees.logevery  <- as.integer(tokens[[linenum]][2])

  linenum <- 12
  stopifnot(tokens[[linenum]][1] == "beastspptree.logevery")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastspptree.logevery <- as.integer(tokens[[linenum]][2])

  linenum <- 13
  stopifnot(tokens[[linenum]][1] == "treeann.gtrees.burnin")
  stopifnot(length(tokens[[linenum]]) == 2)
  treeann.gtrees.burnin  <- as.integer(tokens[[linenum]][2])

  linenum <- 14
  stopifnot(tokens[[linenum]][1] == "treeann.stree.burnin")
  stopifnot(length(tokens[[linenum]]) == 2)
  treeann.stree.burnin <- as.integer(tokens[[linenum]][2])
  
  linenum <- 15
  stopifnot(tokens[[linenum]][1] == "sppdelimanal.burnin")
  stopifnot(length(tokens[[linenum]]) == 2)
  sppdelimanal.burnin <- as.integer(tokens[[linenum]][2])
  
  linenum <- 16
  stopifnot(tokens[[linenum]][1] == "sppdelimanal.collapseheight")
  stopifnot(length(tokens[[linenum]]) == 2)
  sppdelimanal.collapseheight <- as.double(tokens[[linenum]][2])
  
  linenum <- 17
  stopifnot(tokens[[linenum]][1] == "sppdelimanal.simcutoff")
  stopifnot(length(tokens[[linenum]]) == 2)
  sppdelimanal.simcutoff <- as.double(tokens[[linenum]][2])
  
  linenum <- 18
  stopifnot(tokens[[linenum]][1] == "msc.model")
  stopifnot(length(tokens[[linenum]]) == 2)
  msc.model <- tokens[[linenum]][2]
  stopifnot(identical(msc.model, "STACEY")  ||  
            identical(msc.model, "STACEYD"))

  list(beast.priors=beast.priors, 
       beast.collapseheight=beast.collapseheight,
       beastseed=beastseed, 
       beastchainlength=beastchainlength,
       beastscreen.logevery=beastscreen.logevery, 
       beastparams.logevery=beastparams.logevery,
       beastgtrees.logevery=beastgtrees.logevery, 
       beastspptree.logevery=beastspptree.logevery,
       treeann.gtrees.burnin=treeann.gtrees.burnin, 
       treeann.stree.burnin=treeann.stree.burnin,
       sppdelimanal.burnin=sppdelimanal.burnin,
       sppdelimanal.collapseheight=sppdelimanal.collapseheight,
       sppdelimanal.simcutoff=sppdelimanal.simcutoff,
       msc.model=msc.model)
 }









