#                             STACEY
# These are functions called by STACEY_main-GNT.r, STACEY_main-GNT-prior.r,
# (should be STACEY_main-olave.r too...) to make gene trees,
# sequences, UPGMA pdfs, XML, and run BEAST, TreeAnn, SDA, etc.



############################################################################################
########################### the main functions called elsewhere ############################
############################################################################################



make.BEAST2.XMLs <- function(doing.olave, prior.only) {
  nofGNT <- length(CONFIG$configG)
  replicate.sets <- make.replicate.sets()
  for (i in 1:nofGNT) {    
    if (!file.exists(fpath.GNTdpath(CONFIG, i))) {
      dir.create(fpath.GNTdpath(CONFIG, i))
    }    
    # Make the species tree
    spptree <- make.spptree(CONFIG$spptree)
    cat("GNT loop: i =", i, "made species tree\n")
    # add nlins at tips if they're not already there
    # and check they're not upplied twice
    for (nn in 1:length(spptree)) {
      if (spptree[[nn]]$nheight == 0) {
        if (spptree[[nn]]$nlins == -1) {
          stopifnot(CONFIG$configN[i] > 0)
          spptree[[nn]]$nlins <- CONFIG$configN[i]
        } else {
          stopifnot(CONFIG$configN[i] == -1)
        }
      } else {
        spptree[[nn]]$nlins <- -1
      }
    }  
    
    # one-node tree is special case
    if (length(spptree) > 1) {
      Newick.spptree <- spptree.to.Newick(spptree)
      print(Newick.spptree)
      phylo.spptree <- read.tree(text=Newick.spptree)                                                                                          
      
      pdf(file=fpath.spptree.pdf(CONFIG, i))
      plot.phylo(phylo.spptree)
      axisPhylo()
      title("Simulated species tree")
      dev.off()
    }
    
    sink(fpath.spptree(CONFIG, i))
    print.spptree(spptree)
    sink(NULL)
    
    cat("making BEAST2 XML files\n")
    for (r in replicate.sets[[i]]) {
      if (prior.only) {
        stopifnot(r==1)
        make.beast2xml.prior.only(CONFIG, i, spptree, "B")
        make.beast2xml.prior.only(CONFIG, i, spptree, "BN")
        make.beast2xml.prior.only(CONFIG, i, spptree, "BF")
        make.beast2xml.prior.only(CONFIG, i, spptree, "BR")
        make.beast2xml.prior.only(CONFIG, i, spptree, "BNFR")
      } else {
        if (doing.olave) {
          # TODO
        } else {
          make.beast2xml.for.one.GNT.one.replicate(CONFIG, spptree, i, r)
        }
      }
    }
  }
}




make.UPGMA.gtrees <- function() {
  nofGNT <- length(CONFIG$configG)  
  replicate.sets <- make.replicate.sets()
  for (i in 1:nofGNT) {
    for (r in replicate.sets[[i]]) {
      make.upgma.gtrees.for.one.GNT.one.replicate(CONFIG, i, r)
      }
    }
}



run.BEAST2.allreps <- function(java.fpath, BEAST.jar.fpath) {
  nofGNT <- length(CONFIG$configG) 
  replicate.sets <- make.replicate.sets()
  for (i in 1:nofGNT) {
    for (r in replicate.sets[[i]]) {
      run.beast2.for.one.GNT.one.replicate(java.fpath, BEAST.jar.fpath, CONFIG, i, r)
    }
  }
}



run.TreeAnn.allreps <- function(java.fpath, TreeAnn.jar.fpath) {
  nofGNT <- length(CONFIG$configG)  
  replicate.sets <- make.replicate.sets()
  for (i in 1:nofGNT) {
    for (r in replicate.sets[[i]]) {
      run.treeannotator.for.one.GNT.one.replicate(java.fpath, TreeAnn.jar.fpath, CONFIG, i, r)
      }
    }  
}



run.SppDA.make.simmatrix.allreps <- function(java.fpath, SppDA.jar.fpath) {
  if (identical(CONFIG$progparams$msc.model, "STACEYD")) {
    nofGNT <- length(CONFIG$configG)  
    replicate.sets <- make.replicate.sets()
    for (i in 1:nofGNT) {
      for (r in replicate.sets[[i]]) {
        run.speciesdelimanalyser.for.one.GNT.one.replicate(java.fpath, SppDA.jar.fpath, CONFIG, i, r)
      }
    }
    for (i in 1:nofGNT) {
      for (r in replicate.sets[[i]]) {
        make.simmatrix.for.one.GNT.one.replicate(CONFIG, i, r)
      }
    }    
  }
}



#######################################################
####################  low level functions ############


make.genetrees.for.one.GNT.one.replicate <- function(config, spptree, i, r) { 
  # Simulate the gene trees
  cat("making gene trees for GNT", i, "replicate", r, "\n")
  for (g in 1:config$configG[i]) {
    gt <- make.one.genetree.as.Newick(spptree, config$configT[i], getrandomseed(r, g))
    sink(fpath.gtree(config, i, r, g))
    cat(gt)
    sink(NULL)
    phylo.tree <- read.tree(text=gt)
    pdf(file=fpath.gtree.pdf(config, i, r, g), width=9, height=2+0.1*nrow(phylo.tree$edge))
    plot.phylo(phylo.tree, edge.color="grey")
    axisPhylo()
    title(paste("Simulated gene tree: rep", r, "gene", g))
    dev.off()
  }
}


make.sequences.for.one.GNT.one.replicate <- function(config, i, r) { 
  # Simulate the sequence alignments
  cat("making sequence alignments for GNT", i, "replicate", r, "\n")
  for (g in 1:config$configG[i]) {  
    SeqGen.cmd <- paste(seqgenbasecmd, 
                        " -l", config$seqs$len, 
                        " -n1 -z", getrandomseed(r, g), 
                        " ", fpath.gtree(config, i, r, g), 
                        " -y ", fpath.seq(config, i, r, g), 
                        sep="")
    cat("Running seqgen with command:", SeqGen.cmd, "\n")
    system(SeqGen.cmd)
    
    x <- read.dna(file=fpath.seq(config, i, r, g))
    xch <- as.character(x)
    xchlst <- as.list(as.data.frame(t(xch)))
    write.nexus.data(xchlst, file=fpath.seq.nex(config, i, r, g), 
                     format = "dna", datablock=FALSE, interleaved=FALSE)
    
    
  }
}


make.upgma.gtrees.for.one.GNT.one.replicate <- function(config, i, r) { 
  for (g in 1:config$configG[i]) {               
    gseq <- read.dna(file=fpath.seq(config, i, r, g), format = "sequential")
    gmat <- dist.dna(gseq, as.matrix=TRUE)
    upgma.gtree <- upgma(gmat)
    pdf(file=fpath.upgmagtree.pdf(config, i, r, g), width=9, height=2+0.1*nrow(upgma.gtree$edge))
    plot.phylo(upgma.gtree)
    title(main=paste("UPGMA gene tree", g, "GNT", i, "rep", r))
    dev.off()
  }
}


make.beast2xml.for.one.GNT.one.replicate <- function(config, spptree, i, r) {  
  
  make.genetrees.for.one.GNT.one.replicate(config, spptree, i, r)
  
  make.sequences.for.one.GNT.one.replicate(config, i, r) 
  
  sim.options <- list(msc.model=CONFIG$progparams$msc.model,
                      prior.only=FALSE,
                      doing.olave=FALSE,
                      nofgenes=config$configG[i],
                      taxa=collect.taxa(spptree),
                      beast.priors=config$progparams$beast.priors,
                      beast.collapseheight=CONFIG$progparams$beast.collapseheight,
                      seqs.fpathbase=fpath.seqs.fpathbase(config, i, r), 
                      sampledgtrees.fpathbase=fpath.sampledgtrees.fpathbase(config, i, r, ""),
                      sampledspptrees.fpath=fpath.sampledspptrees(config, i, r, ""),
                      sampledparams.fpath=fpath.sampledparams(config, i, r, ""),
                      DBUGTUNE.fpath=fpath.DBUGTUNE(config, i, r),
                      beastchainlength=CONFIG$progparams$beastchainlength,
                      beastparams.logevery=CONFIG$progparams$beastparams.logevery,
                      beastspptree.logevery=CONFIG$progparams$beastspptree.logevery,
                      beastgtrees.logevery=CONFIG$progparams$beastgtrees.logevery,
                      beastscreen.logevery=CONFIG$progparams$beastscreen.logevery)
  # make.beast2xml.from.sim.data() is in STACEY_5beast2xml_toplevel.r
  sink(fpath.beastxml(config, i, r, ""))
  make.beast2xml.from.sim.data(sim.options, "")
  sink(NULL)
}



make.beastxml.for.olave.one.replicate <- function(r, Ne, sptree.shape, numgenes) {  
  
  sim.options <- list(msc.model=CONFIG$progparams$msc.model,
                      prior.only=FALSE,
                      doing.olave=FALSE,
                      nofgenes=config$configG[i],
                      taxa=collect.taxa(spptree),
                      beast.priors=config$progparams$beast.priors,
                      beast.collapseheight=CONFIG$progparams$beast.collapseheight,
                      seqs.fpathbase=fpath.seqs.fpathbase(config, i, r), 
                      sampledgtrees.fpathbase=fpath.sampledgtrees.fpathbase(config, i, r, ""),
                      sampledspptrees.fpath=fpath.sampledspptrees(config, i, r, ""),
                      sampledparams.fpath=fpath.sampledparams(config, i, r, ""),
                      DBUGTUNE.fpath=fpath.DBUGTUNE(config, i, r),
                      beastchainlength=CONFIG$progparams$beastchainlength,
                      beastparams.logevery=CONFIG$progparams$beastparams.logevery,
                      beastspptree.logevery=CONFIG$progparams$beastspptree.logevery,
                      beastgtrees.logevery=CONFIG$progparams$beastgtrees.logevery,
                      beastscreen.logevery=CONFIG$progparams$beastscreen.logevery)
  
  sink(olave.beastxml.fpath(r, Ne, sptree.shape, numgenes))
  make.beast2xml.from.sim.data(sim.options, "")
  sink(NULL)
}



make.beast2xml.prior.only <- function(config, i, spptree, operators.choice) {  
  r <- 1
  sim.options <- list(msc.model=CONFIG$progparams$msc.model,
                      prior.only=TRUE,
                      doing.olave=FALSE,
                      nofgenes=config$configG[i],
                      taxa=collect.taxa(spptree),
                      beast.priors=config$progparams$beast.priors,
                      beast.collapseheight=CONFIG$progparams$beast.collapseheight,
                      seqs.fpathbase=NULL, 
                      sampledgtrees.fpathbase=fpath.sampledgtrees.fpathbase(config, i, r, operators.choice),
                      sampledspptrees.fpath=fpath.sampledspptrees(config, i, r, operators.choice),
                      sampledparams.fpath=fpath.sampledparams(config, i, r, operators.choice),
                      DBUGTUNE.fpath=fpath.DBUGTUNE(config, i, r),
                      beastchainlength=CONFIG$progparams$beastchainlength,
                      beastparams.logevery=CONFIG$progparams$beastparams.logevery,
                      beastspptree.logevery=CONFIG$progparams$beastspptree.logevery,
                      beastgtrees.logevery=CONFIG$progparams$beastgtrees.logevery,
                      beastscreen.logevery=CONFIG$progparams$beastscreen.logevery)
  sink(fpath.beastxml(config, i, r, operators.choice))
  make.beast2xml.from.sim.data(sim.options, operators.choice)
  sink(NULL)
}



beast.cmd.for.one.GNT.one.replicate <- function(java.fpath, BEAST.jar.fpath, resume, config, i, r) {
  if (config$progparams$beastseed < 0) {
    seedarg <- ""
  }
  else {
    if (config$progparams$beastseed > 0) {
      # use supplied seed
      randseed <- config$progparams$beastseed
    } else {
      # use repeatable seed
      randseed <- 1000*i + r
    }
    seedarg <- paste0("-seed ", randseed, " ")
  }
  resumearg <- ""
  if (resume) { resumearg <- "-resume "}
  paste0(java.fpath, " -enableassertions -jar ", BEAST.jar.fpath, " ",
         "-overwrite ",
         "-working ",
         resumearg,
         "-beagle ",
         seedarg,
         fpath.beastxml(config, i, r))
}



run.beast2.for.one.GNT.one.replicate <- function(java.fpath, BEAST.jar.fpath, resume, config, i, r) {
  BEAST.cmd <- beast.cmd.for.one.GNT.one.replicate(config, i, r)
  cat(BEAST.cmd, "\n")
  errorcode <- system(BEAST.cmd)
  stopifnot(errorcode == 0)  # don't seem to work
}



treeannotator.cmds.for.one.GNT.one.replicate <- function(java.fpath, TreeAnn.jar.fpath, config, i, r) {
  TreeAnnotator.cp <-
    paste0(java.fpath, " -jar ", TreeAnn.jar.fpath, " ")
  cmds <- rep("", config$configG[i])
  for (g in 1:config$configG[i]) {
    cmds[g] <- 
      paste0(TreeAnnotator.cp,
             "-burnin ", config$progparams$treeann.gtrees.burnin, " ",
             fpath.sampledgtree(config, i, r, g), " ",
             fpath.mccgtree(config, i, r, g))
    
  } 
  cmds[config$configG[i] + 1] <- paste0(TreeAnnotator.cp,
                                        "-burnin ", config$progparams$treeann.stree.burnin, " ",
                                        fpath.sampledspptrees(config, i, r), " ",
                                        fpath.mccspptree(config, i, r))
  cmds
}



run.treeannotator.for.one.GNT.one.replicate <- function(java.fpath, TreeAnn.jar.fpath, config, i, r) {
  TreeAnnotator.cmds <- treeannotator.cmds.for.one.GNT.one.replicate(config, i, r)
  for (c in 1:length(TreeAnnotator.cmds)) {
    cat(TreeAnnotator.cmds[c], "\n")
    system(TreeAnnotator.cmds[c])
  } 
} 



speciesdelimanalyser.cmd.for.one.GNT.one.replicate <- function(java.fpath, SppDA.jar.fpath, config, i, r) {
  paste0(java.fpath, " -jar ", SppDA.jar.fpath, " ",
         "-burnin ", config$progparams$sppdelimanal.burnin, " ",
         "-collapseheight ", config$progparams$sppdelimanal.collapseheight, " ",
         "-simcutoff ", config$progparams$sppdelimanal.simcutoff, " ",
         fpath.sampledspptrees(config, i, r), " ",
         fpath.clusterings(config, i, r))  
}


run.speciesdelimanalyser.for.one.GNT.one.replicate <- function(java.fpath, SppDA.jar.fpath, config, i, r) {
  SpeciesDelimitationAnalyser.cmd <- speciesdelimanalyser.cmd.for.one.GNT.one.replicate(java.fpath, SppDA.jar.fpath, config, i, r)
  cat(SpeciesDelimitationAnalyser.cmd, "\n")
  system(SpeciesDelimitationAnalyser.cmd)
}



make.simmatrix.for.one.GNT.one.replicate <- function(config, i, r) {
  make.simmatrix(fpath.clusterings(config, i, r),
                 fpath.simmatrix(config, i, r), fpath.simmatrix.pdf(config, i, r)) 
} 



make.replicate.sets <- function() {
  nofGNT <- length(CONFIG$configG)
  replicate.sets <- NULL
  for (i in 1:nofGNT) {
    replicate.sets <- c(replicate.sets, list(CONFIG$configR0[i]:CONFIG$configR1[i]))
  }  
  replicate.sets
}












  




