


make.beast2xml.distribution <- function(sim.options) { 

  catln(2, "<distribution", id("posterior"), spec("util.CompoundDistribution"), ">")
  make.coalescent.distribution(sim.options)
  make.prior.distribution(sim.options)
  make.likelihood.distribution(sim.options)
  catln(2, "</distribution>")
}


make.coalescent.distribution <- function(sim.options) {
  beast.priors <- sim.options$beast.priors
  nofgenes <- sim.options$nofgenes
  bigcomment("The STACEY pops-integrated-out coalescent")
  catln(3, "<distribution",
        id(smcCoalescentID()),
        spec("stacey.PIOMSCoalescentDistribution"), 
        namevalue("tree", IDtoREF(smcTreeID())),
        namevalue("taxonset", IDtoREF(taxonSetOfSetsID())),
        ">")

  #catln(4, idref("smcTree", smcTreeID(), NULL))
  for (g in 1:nofgenes) {
    catln(4, "<geneTree",
          id(geneTreeCoalFactorID(g)),
          spec("stacey.GtreeAndCoalFactor"),
          namevalue("tree", IDtoREF(geneTreeID(g))),
          namevalue("Ploidy", "2.0"), 
          "/>")
  }

  w <- c("1")
  a <- c("3")
  b <- c("2") 
  #   w <- c("1","1")
  #   a <- c("3","3")
  #   b <- c("1.5","2.5")
  for (i in 1:length(w)) {
    catln(4, "<popPriorInvGamma", 
          id(InvGammaComponentID(i)),
          spec("stacey.InverseGammaComponent"),
          namevalue("weight", w[i]), 
          namevalue("alpha", a[i]), 
          namevalue("beta", b[i]), "/>")
  }
  catln(4, idref(popSFID(), popSFID(), NULL))  
  
  catln(3, "</distribution>")
  
}




make.prior.distribution <- function(sim.options) {
  nofgenes <- sim.options$nofgenes
  bigcomment("Prior")
  beast.priors <- sim.options$beast.priors
  
  catln(3, "<distribution", id("prior"), spec("util.CompoundDistribution"), ">")
  commentln("BirthDeathCollapse prior for smcTree")  
  catln(4, "<distribution", 
        id(bdcModelID()),
        spec("stacey.BirthDeathCollapseModel"), 
        namevalue(bdcCollapseHtID(), sim.options$beast.collapseheight), ">")
  
  catln(5, idref("tree", smcTreeID(), NULL))
  catln(5, idref("parameter", bdcGrowthID(), "birthDiffRate"))
  catln(5, idref("parameter", bdcRelDeathID(), "relativeDeathRate"))
  catln(5, idref("parameter", bdcCollapseWtID(), "collapseWeight"))
  catln(5, idref("parameter", bdcOriginHtID(), "originHeight"))      
  catln(4, "</distribution>")
  
  commentln("Priors for BirthDeathCollapse hyperparameters") 
  if (beast.priors$estimate.growthrate) {
    ID <- paste0(bdcGrowthID(), "Prior")
    make.1Dprior(beast.priors$growthrate, ID, IDtoREF(bdcGrowthID()))   
  }

  if (beast.priors$estimate.reldeathrate) {
    ID <- paste0(bdcRelDeathID(), "Prior")
    make.1Dprior(beast.priors$reldeathrate, ID, IDtoREF(bdcRelDeathID()))
  }
  
  if (beast.priors$estimate.collapseweight) {
    ID <- paste0(bdcCollapseWtID(), "Prior")
    make.1Dprior(beast.priors$collapseweight, ID, IDtoREF(bdcCollapseWtID()))
  }
  
  if (beast.priors$estimate.popmean) {
    commentln("Hyper-prior for population scale factor") 
    ID <- paste0(popSFID(), "Prior")
    make.1Dprior(beast.priors$popmean, ID, IDtoREF(popSFID()))
  }
  
  commentln("Priors for relative clock rates for gene trees (after first one)")  
  if (nofgenes > 1) {
    for (g in 2:nofgenes) {
      ID <- paste0(clockRateID(g), "Prior")
      make.1Dprior(beast.priors$relativerates, ID, IDtoREF(clockRateID(g)))   
    }    
  }

  commentln("Priors for HKY kappas")  
  for (g in 1:nofgenes) {
    ID <- paste0(kappaID(g), "Prior")
    prior <- list(type="lnorm", par1="0.0", par2="1.25")
    make.1Dprior(prior, ID, IDtoREF(kappaID(g)))      
  }
  
  catln(3, "</distribution>")
}



make.1Dprior <- function(prior, ID, xREF) {
  catln(4, "<prior", id(ID),
        name("distribution"),
        namevalue("x", xREF), 
        ">")
  make.ParametricDistribution(prior, ID)
  catln(4, "</prior>")
}


make.ParametricDistribution <- function(prior, ID) {
  if (identical(prior$type, "lnorm")) {
    make.lnorm.prior(prior, ID)
  } else if (identical(prior$type, "beta")) {
    make.beta.prior(prior, ID)
  } else if (identical(prior$type, "exp")) {
    stop("not implemented in make.ParametricDistribution()")
    make.exp.prior(prior, ID)
  } else {
    stop("unknown prior type in make.ParametricDistribution()")
  }  
}






make.lnorm.prior <- function(prior, ID) {
  catln(5, "<LogNormal", 
        name("distr"),
        id(paste0("LogNormal.", ID)),
        ">")
  catln(6, "<parameter", name("M"), 
        id(paste0("mean.", ID)),
        namevalue("estimate", "false"),
        paste0(">", prior$par1, "</parameter>")) 
  catln(6, "<parameter", name("S"), 
        id(paste0("sd.", ID)),
        namevalue("estimate", "false"),
        paste0(">", prior$par2, "</parameter>")) 
  
  catln(5, "</LogNormal>") 
}


make.beta.prior <- function(prior, ID) {
  catln(5, "<Beta", 
        name("distr"),
        id(paste0("Beta.", ID)),
        ">")
  catln(6, "<parameter", name("alpha"), 
        id(paste0("alpha.", ID)),
        namevalue("estimate", "false"),
        paste0(">", prior$par1, "</parameter>")) 
  catln(6, "<parameter", name("beta"), 
        id(paste0("beta.", ID)),
        namevalue("estimate", "false"),
        paste0(">", prior$par2, "</parameter>")) 
  
  catln(5, "</Beta>") 
}



make.exp.prior <- function(prior, paramname, g) {
}




make.likelihood.distribution <- function(sim.options) {
  nofgenes <- sim.options$nofgenes
  bigcomment("gene tree likelihoods")
  catln(3, "<distribution", id("likelihood"), spec("util.CompoundDistribution"), ">")
  for (g in 1:nofgenes) {
    make.gtree.lhood(g)
  }  
  catln(3, "</distribution>")
}



make.gtree.lhood <- function(g) {
  commentln(paste0("tree likelihood for gene ", g))  
  catln(4, "<distribution", 
        namevalue("data", IDtoREF(alignmentID(g))),
        id(geneTreeLhoodID(g)),
        namevalue("tree", IDtoREF(geneTreeID(g))),
        spec("TreeLikelihood"), ">")  
  commentln(paste0("site model for gene ", g)) 
  make.sitemodel(g)
  commentln(paste0("branch model for gene ", g)) 
  make.branchmodel(g)  
  catln(4, "</distribution>")
}


make.sitemodel <- function(g) {
  siteID <- paste0("SiteModel.", g)
  muID <- paste0("mutationRate.", g)
  gamID <- paste0("siteHetGamma.", g)
  invID <- paste0("proportionInvariant.", g)
  catln(5, "<siteModel", id(siteID), spec("SiteModel"), ">")
  catln(6, "<parameter", namevalue("estimate", "false"), id(muID), name("mutationRate"),
        paste0(">1.0</parameter>"))
  catln(6, "<parameter", namevalue("estimate", "false"), id(gamID), name("shape"),
        paste0(">1.0</parameter>"))
  catln(6, "<parameter", namevalue("estimate", "false"), id(invID), name("proportionInvariant"),
        namevalue("lower", "0.0"), namevalue("upper", "1.0"), 
        paste0(">0.0</parameter>"))
   
  hkyID <- paste0("hky.", g)
  freqsID <- paste0("estimatedFreqs.", g)
  catln(6, "<substModel", id(hkyID), namevalue("kappa", IDtoREF(kappaID(g))), spec("HKY"), ">")
  catln(7, "<frequencies", id(freqsID), namevalue("frequencies", IDtoREF(frequenciesID(g))), spec("Frequencies"), "/>")
  catln(6, "</substModel>")
  catln(5, "</siteModel>")
}

make.branchmodel <- function(g) {
  scID <- paste0("StrictClock.", g)
  if (g == 1) {
    # in g==1 case make a new fixed parameter
    catln(5, "<branchRateModel", 
          id(scID), 
          spec("beast.evolution.branchratemodel.StrictClockModel"), ">")
    catln(6, "<parameter", 
          namevalue("estimate", "false"), 
          id(clockRateID(g)), 
          name("clock.rate"),
          paste0(">1.0</parameter>"))
    catln(5, "</branchRateModel>")
  } else {
    # else refer to existing clock rate
    catln(5, "<branchRateModel", 
          id(scID), 
          spec("beast.evolution.branchratemodel.StrictClockModel"), 
          namevalue("clock.rate", IDtoREF(clockRateID(g))), ">")
    catln(5, "</branchRateModel>")
  }

}
  


  