


make.beast2xml.operators <- function(sim.options, operators.choice, nsmcs, totalnofseqs) {
  nofgenes <- sim.options$nofgenes
  
  total.smsctree.weight <- 200 + 40*nofgenes
  nudge <- grepl("N", operators.choice)
  focus <- grepl("F", operators.choice)
  regraft <- grepl("R", operators.choice)
  op.count <- 1
  if (nudge) { op.count <- op.count + 1 }
  if (focus) { op.count <- op.count + 1 }
  if (regraft) { op.count <- op.count + 1 }
  weight.per.op <- as.integer(ceiling (total.smsctree.weight/op.count))
  commentln("smcTree operator")
  make.NodeReheight(nofgenes, weight.per.op)
  commentln("smcTree and gene trees operators")
  if (nudge) { make.NodeNudge(nofgenes, weight.per.op) }
  if (regraft) { make.CoordinatedPruneRegraft(nofgenes, weight.per.op) }
  if (focus) { make.FocusedScaler(nofgenes, weight.per.op) }
  
  beast.priors <- sim.options$beast.priors
    
  commentln("stretch and squeeze everything") 
  make.upGrowthClocks.downPopsHeights(nofgenes, 30 + 5*nofgenes, 
                                      beast.priors$estimate.growthrate, 
                                      beast.priors$estimate.popmean) 
  
  commentln("a bunch of scalers for smcTree (hyper-)parameters") 
  if (beast.priors$estimate.popmean) 
    { make.general.scaleOperator(popSFScalerID(), popSFID(), 0.75, 5+nofgenes) }
  if (beast.priors$estimate.growthrate) 
    { make.general.scaleOperator(bdcGrowthScalerID(), bdcGrowthID(), 0.75, 5+nofgenes) }
  if (beast.priors$estimate.reldeathrate)
    { make.general.scaleOperator(bdcReldeathScalerID(), bdcRelDeathID(), 0.75, 5+nofgenes) }
  if (beast.priors$estimate.collapseweight) {
    make.general.scaleOperator(bdcCollapseWtScalerID(), bdcCollapseWtID(), 0.75, 5+nofgenes)
  }
 
  make.general.scaleOperator(bdcOriginHtScalerID(), bdcOriginHtID(), 0.75, 5+nofgenes)
  
  gtwm <- g.t.w.mul(nofgenes, totalnofseqs)
  for (g in 1:nofgenes) {
    bigcomment(paste0("operators for gene tree ", g))   
    if (nsmcs > 2) { 
      commentln(paste0("topology-changing ops for gene tree ", g)) 
      make.subtreeSlide(g, g.t.w(15, gtwm)) 
      make.narrowExchange(g, g.t.w(15, gtwm)) 
      make.wideExchange(g, g.t.w(4, gtwm)) 
      make.wilsonBalding(g, g.t.w(4, gtwm)) 
    }
    commentln(paste0("height-only-changing ops for gene tree ", g))
    make.uniformOperator.internalNodeHeights(g, g.t.w(30, gtwm)) 
    make.scaleOperator.allHeights(g, g.t.w(4, gtwm)) 
    make.scaleOperator.rootHeight(g, g.t.w(4, gtwm)) 
    commentln(paste0("op for heights and relative clock rate for gene tree ", g)) 
    make.upRate.downHeights(g, g.t.w(30, gtwm))
    if (g > 1) {
      commentln(paste0("op for relative clock rate for gene tree ", g)) 
      make.general.scaleOperator(clockRateScalerID(g), clockRateID(g), 0.75, g.t.w(10, gtwm))
    }
    commentln(paste0("kappas and frequencies ops for gene tree ", g)) 
    make.general.scaleOperator(kappaScalerID(g), kappaID(g), 0.5, g.t.w(2, gtwm))
    make.deltaExchange.frequencies(g, g.t.w(2, gtwm))
  }  
}  
  





make.NodeReheight <- function(nofgenes, weight) {
  catln(2, "<operator", 
        id(nodeReheightID()), 
        spec("NodeReheight"),
        namevalue("taxonset", IDtoREF(taxonSetOfSetsID())),
        namevalue("tree", IDtoREF(smcTreeID())),
        namevalue("weight", weight), ">")
  for (g in 1:nofgenes) {
    catln(3, idref("tree", geneTreeID(g), "genetree"))
  }
  catln(2, "</operator>")
} 



make.NodeNudge <- function(nofgenes, weight) {
  catln(2, "<operator", 
        id(nodesNudgeID()), 
        spec("stacey.NodesNudge"),
        namevalue("weight", weight), ">")
  catln(3, idref("smcTree", smcTreeID(), NULL))
  for (g in 1:nofgenes) {
    catln(3, idref("tree", geneTreeID(g), "geneTree"))
  }
  catln(2, "</operator>")
} 


make.CoordinatedPruneRegraft <- function(nofgenes, weight) {
  catln(2, "<operator", 
        id(coordinatedPruneRegraftID()), 
        spec("stacey.CoordinatedPruneRegraft"),
        namevalue("weight", weight), ">")
  catln(3, idref("smcTree", smcTreeID(), NULL))
  for (g in 1:nofgenes) {
    catln(3, idref("tree", geneTreeID(g), "geneTree"))
  }
  catln(2, "</operator>")
} 


make.FocusedScaler <- function(nofgenes, weight) {
  catln(2, "<operator", 
        id(focusedScalerID()), 
        spec("stacey.FocusedNodeHeightScaler"),
        namevalue("weight", weight), ">")
  catln(3, idref("smcTree", smcTreeID(), NULL))
  for (g in 1:nofgenes) {
    catln(3, idref("tree", geneTreeID(g), "geneTree"))
  }
  catln(2, "</operator>")
} 



make.upGrowthClocks.downPopsHeights <- function(nofgenes, weight, estimate.bdcGrowth, estimate.popmean) {
  #upGrowthClocks.downPopsHeightsID
  
  catln(2, "<operator", 
        id(upGrowthClocks.downPopsHeightsID()), 
        spec("UpDownOperator"),
        namevalue("weight", weight), 
        namevalue("scaleFactor", "0.75"), ">")
  if (estimate.bdcGrowth) {
    catln(3, idref("parameter", bdcGrowthID(), "up"))
  }
  if (nofgenes > 1) {
    for (g in 2:nofgenes) {
      catln(3, idref("parameter", clockRateID(g), "up"))
    }
  }
  if (estimate.popmean) {
    catln(3, idref("parameter", popSFID(), "down"))
  }
  catln(3, idref("tree", smcTreeID(), "down"))
  for (g in 1:nofgenes) {
    catln(3, idref("tree", geneTreeID(g), "down"))
  }  
  catln(2, "</operator>")  
}



make.subtreeSlide <- function(g, weight) {
  catln(2, "<operator", 
        id(subtreeSlideID(g)),  
        spec("SubtreeSlide"),
        namevalue("tree", IDtoREF(geneTreeID(g))),
        namevalue("weight", weight), "/>")
}


make.narrowExchange <- function(g, weight) {
  catln(2, "<operator", 
        id(narrowID(g)),  
        spec("Exchange"),
        namevalue("tree", IDtoREF(geneTreeID(g))),
        namevalue("weight", weight), "/>")  
}


make.wideExchange <- function(g, weight) {
  catln(2, "<operator", 
        id(wideID(g)),  
        spec("Exchange"),
        namevalue("isNarrow", "false"),
        namevalue("tree", IDtoREF(geneTreeID(g))),
        namevalue("weight", weight), "/>")  
}


make.wilsonBalding <- function(g, weight) {
  catln(2, "<operator", 
        id(WilsonBaldingID(g)),  
        spec("WilsonBalding"),
        namevalue("tree", IDtoREF(geneTreeID(g))),
        namevalue("weight", weight), "/>") 
}


make.uniformOperator.internalNodeHeights <- function(g, weight) {
  catln(2, "<operator", 
        id(uniformID(g)),  
        spec("Uniform"),
        namevalue("tree", IDtoREF(geneTreeID(g))),
        namevalue("weight", weight), "/>") 
}


make.scaleOperator.rootHeight <- function(g, weight) {
  catln(2, "<operator", 
        id(treeRootScalerID(g)),  
        spec("ScaleOperator"),
        namevalue("rootOnly", "true"),
        namevalue("tree", IDtoREF(geneTreeID(g))),
        namevalue("scaleFactor", "0.5"),
        namevalue("weight", weight), "/>") 
}


make.scaleOperator.allHeights <- function(g, weight) {
  catln(2, "<operator", 
        id(treeScalerID(g)),  
        spec("ScaleOperator"),
        namevalue("tree", IDtoREF(geneTreeID(g))),
        namevalue("scaleFactor", "0.5"),
        namevalue("weight", weight), "/>") 
}


make.upRate.downHeights <- function(g, weight) {
  catln(2, "<operator", 
        id(upClock.downHeightsID(g)),  
        spec("UpDownOperator"),
        namevalue("scaleFactor", "0.75"),
        namevalue("weight", weight), ">") 
  if (g > 1) {
    catln(3, idref("parameter", clockRateID(g), "up"))
  }  
  catln(3, idref("tree", geneTreeID(g), "down"))
  catln(2, "</operator>")
}







make.deltaExchange.frequencies <- function(g, weight) {
  frequenciesExchangerID
  #     <operator delta="0.01" id="FrequenciesExchanger.s:29" spec="DeltaExchangeOperator" weight="0.1">
  #     <parameter idref="freqParameter.s:29"/>
  #     </operator>
  #     
  ID <- paste0("FrequenciesExchanger.", g)
  catln(2, "<operator", 
        id(ID),
        namevalue("delta", 0.01),
        namevalue("weight", weight), 
        spec("DeltaExchangeOperator"), ">")
  catln(3, idref("parameter", frequenciesID(g), NULL))
  catln(2, "</operator>")  
}



make.general.scaleOperator <- function(ID, param, sfactor, weight) {
  catln(2, "<operator", id(ID),
        spec("ScaleOperator"), 
        namevalue("scaleFactor", sfactor),
        namevalue("weight", weight), ">")
  
  catln(3, idref("parameter", param, NULL))
  catln(2, "</operator>")
}	



g.t.w.mul <- function(nofgenes, totalnofseqs) {
  relativeweight <- totalnofseqs / (100*nofgenes)
  1 + relativeweight
}


g.t.w <- function(wt, mul) {
  wt <- wt*mul
  if (wt < 1) {
    1
  } else {
    as.integer(round(wt*mul))
  } 
}




