library(coda)
library(MASS)


fpathbase <- "C:/Users/Work/AAA/Programming/ProgramOutput/STACEY/P/Pg1t10/P4STSg1t10r1s42"


read.clusterings <- function(ops) {
  x <- read.table(paste0(fpathbase, ops, "-SDA.txt"), header=TRUE)
  x
  }






prob.partition.shape <- function(x) {
  r <- length(x)
  n <- sum(x)
  s <- rep(0,n)
  for (i in 1:n) {
    w <- which(x == i)
    s[i] <- length(w)
  }
  z <- factorial(r)
  z <- z / choose(n-1, r-1)
  for (i in 1:n) {
    z <- z / factorial(s[i])
  }
  z
}






make.matrix.of.clusterings <- function() {
  clusterings <- matrix(0, nrow=22, ncol=8)
  
  clusterings[1,1] <- c(8)
  
  clusterings[2,1:2] <- c(7,1)
  clusterings[3,1:2] <- c(6,2)
  clusterings[4,1:2] <- c(5,3)
  clusterings[5,1:2] <- c(4,4)
  
  clusterings[6,1:3] <- c(6,1,1)
  clusterings[7,1:3] <- c(5,2,1)
  clusterings[8,1:3] <- c(4,3,1)
  clusterings[9,1:3] <- c(4,2,2)
  clusterings[10,1:3] <- c(3,3,2)
  
  clusterings[11,1:4] <- c(5,1,1,1)
  clusterings[12,1:4] <- c(4,2,1,1)
  clusterings[13,1:4] <- c(3,3,1,1)
  clusterings[14,1:4] <- c(3,2,2,1)
  clusterings[15,1:4] <- c(2,2,2,2)
  
  clusterings[16,1:5] <- c(4,1,1,1,1)
  clusterings[17,1:5] <- c(3,2,1,1,1)
  clusterings[18,1:5] <- c(2,2,2,1,1)
  
  clusterings[19,1:6] <- c(3,1,1,1,1,1)
  clusterings[20,1:6] <- c(2,2,1,1,1,1)
  
  clusterings[21,1:7] <- c(2,1,1,1,1,1,1)
  
  clusterings[22,1:8] <- c(1,1,1,1,1,1,1,1)
 
  rownames(clusterings) <- c("8", 
                             "7+1",         "6+2",       "5+3",     "4+4", 
                             "6+1+1",       "5+2+1",     "4+3+1",   "4+2+2",   "3+3+2",
                             "5+1+1+1",     "4+2+1+1",   "3+3+1+1", "3+2+2+1", "2+2+2+2", 
                             "4+1+1+1+1",   "3+2+1+1+1", "2+2+2+1+1", 
                             "3+1+1+1+1+1", "2+2+1+1+1+1", 
                             "2+1+1+1+1+1+1", 
                             "1+1+1+1+1+1+1+1")
  
  clusterings
}




getclusterings.probs.for.ops <- function(ops) {
  c.matrix <- make.matrix.of.clusterings()
  names <- rownames(c.matrix)
  
  counts <- rep(0, nrow(c.matrix))
  x <- read.clusterings(ops)
  for (i in 1:nrow(x)) {
    nc <- x[i,"nclusters"]
    ctr <- x[i,-(1:4)]
    sizecounts <- rep(0, nc)
    for (j in 1:nc) {
      sizecounts[j] <- length(which(ctr==j))
    }
    sizecounts <- sort(sizecounts, decreasing=TRUE)
    name <- ""
    for (j in 1:nc) {
      name <-  paste0(name, sizecounts[j])
      if (j < nc) {
        name <-  paste0(name, "+")
      }
    }
    m <- which(names==name)
    #cat(name, m, "   ")
    stopifnot(length(m) == 1)
    counts[m] <- counts[m] + x[i,"count"]
  }
  print(counts)
  counts / sum(counts)
}




get.and.plot.clusterings.probs.for.ops <- function(ops) {
  c.matrix <- make.matrix.of.clusterings()
  names <- rownames(c.matrix)
  probs <- rep(0, nrow(c.matrix))
  for (i in 1:nrow(c.matrix)) {
    x <- c.matrix[i,]
    probs[i] <- prob.partition.shape(x[x>0])
  }
  
  counts <- getclusterings.probs.for.ops(ops)
  par(mar=c(7, 4, 2, 0) + 0.1, cex.axis=0.7)
  barplot(counts, space=0, ylim=c(0,1.2*max(counts)),
          ylab="probability",
          names.arg=names, las=2, )
  mtext(text=paste0("Operators ", ops), line=0)
  points((1:22)-0.5, probs/8, pch=18)
}


pdf(file=paste0(fpathbase, "clustersizes.pdf"), width=7, height=7)
par(mfcol=c(2,2))
get.and.plot.clusterings.probs.for.ops("B")
get.and.plot.clusterings.probs.for.ops("BF")
get.and.plot.clusterings.probs.for.ops("BN")
get.and.plot.clusterings.probs.for.ops("BR")
dev.off()



pdf(file=paste0(fpathbase, "clustersizesBR.pdf"), width=4, height=4)
get.and.plot.clusterings.probs.for.ops("BR")
dev.off()





