library(coda)
library(MASS)


#fpathbase <- "C:/Users/Work/AAA/Programming/ProgramOutput/STACEY/Q/Qg1t10/Q4LOGg1t10r1s42"
fpathbase <- "C:/Users/Work/AAA/Programming/ProgramOutput/STACEY/P/Pg1t10/P4LOGg1t10r1s42"


read.log <- function(ops) {
  x <- read.table(paste0(fpathbase, ops, ".txt"), header=TRUE)
  burnin <- as.integer(nrow(x)/10)
  xb <- x[burnin:nrow(x),]
  xx <- as.mcmc(xb)
  xx
  }

# MIGHT BE USEFUL ELSEWHERE
# make.density <- function(xx, name, min=-Inf, max=Inf) {
#   x <- xx[,name]
#   ESS <- effectiveSize(x)
#   if (ESS < 1) { ESS <- 1 }
#   dens <- density(x, adjust=(nrow(xx) / ESS) ^ 0.2)
#   wsmall <- which(dens$x < min)
#   wbig <- which(dens$x > max)
#   omit <- c(wsmall, wbig)
#   if (length(omit) > 0) {
#     xinrange <- dens$x[-omit]
#     yinrange <- dens$y[-omit]
#   } else {
#     xinrange <- dens$x
#     yinrange <- dens$y
#   }
#   ninrange <- length(xinrange)
#   ysmall <- rev(dens$y[wsmall])
#   if (length(ysmall) > ninrange) {
#     ysmall <- ysmall[1:ninrange]
#   } else {
#     ysmall <- c(ysmall, rep(0, ninrange - length(ysmall)))
#   }
#   ybig <- rev(dens$y[wbig])
#   if (length(ybig) > ninrange) {
#     ybig <- ybig[(length(ybig)-ninrange+1):(length(ybig))]
#   } else {
#     ybig <- c(rep(0, ninrange - length(ybig)), ybig)
#   }  
#   yinrange <- yinrange + ysmall + ybig
#   dd <- list(x=xinrange, y=yinrange, name=name, bw=dens$bw, ESS=ESS)
#   dd
# }




plot.truehist <- function(xx, name, ops) {
  x <- xx[,name]
  ESS <- effectiveSize(x)
  if (ESS < 1) { ESS <- 1 }
  truehist(x, main = paste0("Operators ", ops),
       xlab=paste0(name, " (ESS=", round(ESS), ")"),
       ylab="density",
       col="grey")
}


plot.NClusters <- function(xx) {
  x <- xx[,"NClusters"]
  n <- max(x)
  ESS <- effectiveSize(x)
  hist(x, 
       breaks=seq(from=0.5,to=n+0.5,by=1), 
       main="", 
       xlab=paste0("NClusters", " (ESS=", round(ESS), ")"))
}


plot.hists.for.ops <- function(ops) {
  xx <- read.log(ops)
  layout.m <- matrix(1:9, nrow=3, ncol=3, byrow=TRUE)
  layout(mat=layout.m)
  plot.truehist(xx, "posterior", ops)
  plot.truehist(xx, "likelihood", ops)
  plot.truehist(xx, "prior", ops)
  plot.truehist(xx, "smcCoalescent", ops)
  plot.truehist(xx, "BirthDeathCollapseModel", ops)
  plot.truehist(xx, "bdcCollapseWeight.smcTree", ops)
  plot.truehist(xx, "bdcOriginHeight.smcTree", ops)
  plot.truehist(xx, "PopSize", ops)
  plot.NClusters(xx)
}




get.and.make.pdf.hists.for.ops <- function(ops) {
  pdf(file=paste0(fpathbase, "hists", ops, ".pdf"), width=7, height=9)
  plot.hists.for.ops(ops)
  dev.off()
}



plot.traces.for.ops <- function(ops) {
  xx <- read.log(ops)
  xyplot(xx[,c("posterior", "prior", "smcCoalescent", 
               "bdcCollapseWeight.smcTree", "NClusters", "bdcOriginHeight.smcTree")],
         main=paste0("Operators ", ops))
}


print.summaries.for.ops <- function(ops) {
  cat("Operators", ops, "\n")
  xx <- read.log(ops)
  print(summary(xx))
  cat("\n\n\n")
}


print.summaries <- function(ops) {
  print.summaries.for.ops("B")
  print.summaries.for.ops("BF")
  print.summaries.for.ops("BN")
  print.summaries.for.ops("BR")
}


get.and.make.pdf.hists.for.ops("B")
get.and.make.pdf.hists.for.ops("BF")
get.and.make.pdf.hists.for.ops("BN")
get.and.make.pdf.hists.for.ops("BR")

sink(paste0(fpathbase, "summaries.txt"))
print.summaries()
sink(NULL)



