library(MASS)
library(ape)


fpathbase <- "C:/Users/Work/AAA/Programming/ProgramOutput/STACEY/Q/Qg1t10/Q4STSg1t10r1s42"
BIRTHRATE <- 100

# theoretical height of k'th speciation in tree under Yule model,
# with n tips, from Stadler, "The conditioned reconstructed process".
# BIRTHRATE is lambda. First one is root.
node.height.density <- function(x, n, k) {
  x <- x * BIRTHRATE
  y <- k + 1
  y <- y * choose(n, k+1)
  y <- y * exp(-(k+1)*x)
  y <- y * (1 - exp(-x))^(n-k-1)
  y * BIRTHRATE
}

# find all heights using branching.times() [which seems very slow btw]
# and sort them.
read.trees.remove.burnin <- function(fpath) {
  alltrees <- read.nexus(file=fpath)
  nalltrees <- length(alltrees)
  burnin <- as.integer(nalltrees/10)
  trees <- alltrees[burnin:nalltrees]  
  trees
}



# find all heights using branching.times() [which seems very slow btw]
# and sort them.
get.node.heights <- function(fpath) {
  trees <- read.trees.remove.burnin(fpath) 
  ntrees <- length(trees)
  bts <- matrix(0, ncol=trees[[1]]$Nnode, nrow=ntrees)
  for (t in 1:ntrees) {
    if (t %% 100 == 0) cat(t, " ")
    if (t %% 1000 == 0) cat("\n")
    bts[t,] <- sort(branching.times(trees[[t]]), decreasing=TRUE)
  }
  bts
}


# plot histogram and true density for one set of operators
plot.hists <- function(bts, ops) {
  nplots <- ncol(bts)
  xx <- seq(from=0, to=5/BIRTHRATE, by=.01/BIRTHRATE)
  ncol <- as.integer(round(sqrt(nplots)))
  nrow <- as.integer(ceiling(nplots/ncol))
  layout(mat<-matrix(c(1:nplots, rep(0,ncol*nrow-nplots)), 
                     nrow=nrow, ncol=ncol, byrow=TRUE))
  for (k in 1:nplots) {
    truehist(bts[,k], 
             main=paste0("Operators ", ops), 
             xlab=paste0("height of speciation ", k),
             ylab="density",
             col="grey")
    lines(x=xx, y=node.height.density(xx, nplots+1, k))
  }
}


get.and.plot.node.heights.for.ops <- function(ops) {
  bts <- get.node.heights(paste0(fpathbase, ops, ".txt"))
  plot.hists(bts, ops)
}


get.and.make.pdf.node.heights.for.ops <- function(ops) {
  pdf(file=paste0(fpathbase, "heights", ops, ".pdf"), width=7, height=9)
  get.and.plot.node.heights.for.ops(ops)
  dev.off()
}



# plot histogram and true density for one set of operators, one speciation
plot.hists.kth.speciation <- function(bts, ops, k) {
  nplots <- ncol(bts)
  xx <- seq(from=0, to=5/BIRTHRATE, by=.01/BIRTHRATE)
  truehist(bts[,k], 
           main=paste0("Operators ", ops), 
           xlab=paste0("height of speciation ", k),
           ylab="density",
           col="grey")
  lines(x=xx, y=node.height.density(xx, nplots+1, k))
}


get.and.plot.node.heights.for.ops.kth.speciation <- function(ops, k) {
  bts <- get.node.heights(paste0(fpathbase, ops, ".txt"))
  plot.hists.kth.speciation(bts, ops, k)
}


get.and.make.pdf.node.heights.for.ops.kth.speciation <- function(ops, k) {
  pdf(file=paste0(fpathbase, "heights", ops, "speciation", k, ".pdf"), width=4.2, height=5.4)
  get.and.plot.node.heights.for.ops.kth.speciation(ops, k)
  dev.off()
}






#########################################################################

nodeinfo.to.topology4 <- function(nodeinfo, node, topol) {
  stopifnot(nodeinfo[node,5] == 4)
  if (nodeinfo[node,3] == 2) {
    return (paste0(topol, "42"))
  } else {
    return (paste0(topol, "41"))
  }
  stopifnot(FALSE)
}



nodeinfo.to.topology44 <- function(nodeinfo, node1, node2, topol) {
  stopifnot(nodeinfo[node1,5] == 4)
  stopifnot(nodeinfo[node2,5] == 4)
  if (nodeinfo[node1,3] == 2) {
    if (nodeinfo[node2,3] == 2) {
      return (paste0(topol, "42+42"))
    } else {
      return (paste0(topol, "41+42"))
    }
  } else {
    if (nodeinfo[node2,3] == 2) {
      return (paste0(topol, "41+42"))
    } else {
      return (paste0(topol, "41+41"))
    }
  }  
  
  
  topol1 <- nodeinfo.to.topology4(nodeinfo, node1, topol)
  topol2 <- nodeinfo.to.topology4(nodeinfo, node2, topol)
  if (topol1 == topol2) {
    
  }
  if (nodeinfo[node1,3] == 1) {
    topol2 <- nodeinfo.to.topology4(nodeinfo, node2, topol)
    topol <- paste0(topol, "+")
    topol <- nodeinfo.to.topology4(nodeinfo, node1, topol)
  } else {
    topol <- nodeinfo.to.topology4(nodeinfo, node1, topol)
    topol <- paste0(topol, "+")
    topol <- nodeinfo.to.topology4(nodeinfo, node2, topol)    
  }
  return (topol)
}



nodeinfo.to.topology5 <- function(nodeinfo, node, topol) {
  stopifnot(nodeinfo[node,5] == 5)
  if (nodeinfo[node,3] == 4) {
    topol <- paste0(topol, "51,")
    return (nodeinfo.to.topology4(nodeinfo, nodeinfo[node,1], topol))
  }
  if (nodeinfo[node,3] == 3) {
    topol <- paste0(topol, "52")
    return (topol)
  }
  if (nodeinfo[node,3] == 2) {
    topol <- paste0(topol, "52")
    return (topol)
  }
  if (nodeinfo[node,3] == 1) {
    topol <- paste0(topol, "51,")
    return (nodeinfo.to.topology4(nodeinfo, nodeinfo[node,2], topol))
  }
  stopifnot(FALSE)
}

nodeinfo.to.topology6 <- function(nodeinfo, node, topol) {
  stopifnot(nodeinfo[node,5] == 6)
  if (nodeinfo[node,3] == 5) {
    topol <- paste0(topol, "61,")
    return (nodeinfo.to.topology5(nodeinfo, nodeinfo[node,1], topol))
  }
  if (nodeinfo[node,3] == 4) {
    topol <- paste0(topol, "62,")
    return (nodeinfo.to.topology4(nodeinfo, nodeinfo[node,1], topol))
  }
  if (nodeinfo[node,3] == 3) {
    topol <- paste0(topol, "63")
    return (topol)
  }
  if (nodeinfo[node,3] == 2) {
    topol <- paste0(topol, "62,")
    return (nodeinfo.to.topology4(nodeinfo, nodeinfo[node,2], topol))
  }
  if (nodeinfo[node,3] == 1) {
    topol <- paste0(topol, "61,")
    return (nodeinfo.to.topology5(nodeinfo, nodeinfo[node,2], topol))
  }
  stopifnot(FALSE)
}


nodeinfo.to.topology7 <- function(nodeinfo, node, topol) {
  stopifnot(nodeinfo[node,5] == 7)
  if (nodeinfo[node,3] == 6) {
    topol <- paste0(topol, "71,")
    return (nodeinfo.to.topology6(nodeinfo, nodeinfo[node,1], topol))
  }
  if (nodeinfo[node,3] == 5) {
    topol <- paste0(topol, "72,")
    return (nodeinfo.to.topology5(nodeinfo, nodeinfo[node,1], topol))
  }
  if (nodeinfo[node,3] == 4) {
    topol <- paste0(topol, "73,")
    return (nodeinfo.to.topology4(nodeinfo, nodeinfo[node,1], topol))
  }
  if (nodeinfo[node,3] == 3) {
    topol <- paste0(topol, "73,")
    return (nodeinfo.to.topology4(nodeinfo, nodeinfo[node,2], topol))
  }
  if (nodeinfo[node,3] == 2) {
    topol <- paste0(topol, "72,")
    return (nodeinfo.to.topology5(nodeinfo, nodeinfo[node,2], topol))
  }
  if (nodeinfo[node,3] == 1) {
    topol <- paste0(topol, "71,")
    return (nodeinfo.to.topology6(nodeinfo, nodeinfo[node,2], topol))
  }
  stopifnot(FALSE)
}




nodeinfo.to.topology8 <- function(nodeinfo, node, topol) {
  stopifnot(nodeinfo[node,5] == 8)
  if (nodeinfo[node,3] == 7) {
    topol <- paste0(topol, "81,")
    return (nodeinfo.to.topology7(nodeinfo, nodeinfo[node,1], topol))
  }
  if (nodeinfo[node,3] == 6) {
    topol <- paste0(topol, "82,")
    return (nodeinfo.to.topology6(nodeinfo, nodeinfo[node,1], topol))
  }
  if (nodeinfo[node,3] == 5) {
    topol <- paste0(topol, "83,")
    return (nodeinfo.to.topology5(nodeinfo, nodeinfo[node,1], topol))
  }
  if (nodeinfo[node,3] == 4) {
    topol <- paste0(topol, "84,")
    return (nodeinfo.to.topology44(nodeinfo, nodeinfo[node,1], nodeinfo[node,2], topol))
  }
  if (nodeinfo[node,3] == 3) {
    topol <- paste0(topol, "83,")
    return (nodeinfo.to.topology5(nodeinfo, nodeinfo[node,2], topol))
  }
  if (nodeinfo[node,3] == 2) {
    topol <- paste0(topol, "82,")
    return (nodeinfo.to.topology6(nodeinfo, nodeinfo[node,2], topol))
  }
  if (nodeinfo[node,3] == 1) {
    topol <- paste0(topol, "81,")
    return (nodeinfo.to.topology7(nodeinfo, nodeinfo[node,2], topol))
  }
  stopifnot(FALSE)
}



poedges.to.nodeinfo <- function(po.edge, inode.count) {
  edge.count <- nrow(po.edge)
  tip.count <- inode.count + 1
  nodeinfo <- matrix(0, nrow=edge.count+1, ncol=5)
  # cols are lft, rgt, lftsize, rgtsize, size
  for (n in 1:(edge.count+1)) {
    if (n <= edge.count) {
      i <- po.edge[n,2]
    } else {
      i <- po.edge[edge.count, 1]
    }
    
    
    if (i <= tip.count) {
      nodeinfo[n,1] <- -1
      nodeinfo[n,2] <- -1
      nodeinfo[n,3] <- 0
      nodeinfo[n,4] <- 0
      nodeinfo[n,5] <- 1
    } else {
      children <- which(po.edge[,1] == i)
      stopifnot(length(children)==2)
      nodeinfo[n,1] <- children[1]
      nodeinfo[n,2] <- children[2]
      nodeinfo[n,3] <- nodeinfo[ children[1], 5 ]
      nodeinfo[n,4] <- nodeinfo[ children[2], 5 ]
      nodeinfo[n,5] <- nodeinfo[n,3] + nodeinfo[n,4]
    }
  } 
  nodeinfo
  
}



eighttip.topologies <- matrix(c(
  c("81,71,61,51,41", "(*,(*,(*,(*,U))))", (2/7)*(2/6)*(2/5)*(2/4)*(2/3)),
  c("81,71,61,51,42", "(*,(*,(*,(*,B))))", (2/7)*(2/6)*(2/5)*(2/4)*(1/3)),
  c("81,71,61,52",    "(*,(*,(*,(C,T))))", (2/7)*(2/6)*(2/5)*(2/4)),
  c("81,71,62,41",    "(*,(*,(C,U)))",     (2/7)*(2/6)*(2/5)*(2/3)),
  c("81,71,62,42",    "(*,(*,(C,B)))",     (2/7)*(2/6)*(2/5)*(1/3)),
  c("81,71,63",       "(*,(*,(T,T)))",     (2/7)*(2/6)*(1/5)),
  c("81,72,51,41",    "(*,(C,(*,U)))",     (2/7)*(2/6)*(2/4)*(2/3)),
  c("81,72,51,42",    "(*,(C,(*,B)))",     (2/7)*(2/6)*(2/4)*(1/3)),
  c("81,72,52",       "(*,(C,(C,T)))",     (2/7)*(2/6)*(2/4)),
  c("81,73,41",       "(*,(T,U))",         (2/7)*(2/6)*(2/3)),
  c("81,73,42",       "(*,(T,B))",         (2/7)*(2/6)*(1/3)),
  c("82,61,51,41",    "(C,(*,(*,U)))",     (2/7)*(2/5)*(2/4)*(2/3)),
  c("82,61,51,42",    "(C,(*,(*,B)))",     (2/7)*(2/5)*(2/4)*(1/3)),
  c("82,61,52",       "(C,(*,(C,T)))",     (2/7)*(2/5)*(2/4)),
  c("82,62,41",       "(C,(C,U))",         (2/7)*(2/5)*(2/3)),
  c("82,62,42",       "(C,(C,B))",         (2/7)*(2/5)*(1/3)),
  c("82,63",          "(C,(T,T))",         (2/7)*(1/5)),
  c("83,51,41",       "(T,(*,U))",         (2/7)*(2/4)*(2/3)),
  c("83,51,42",       "(T,(*,B))",         (2/7)*(2/4)*(1/3)),
  c("83,52",          "(T,(C,T))",         (2/7)*(2/4)),
  c("84,41+41",       "(U,U)",             (1/7)*(2/3)*(2/3)),
  c("84,41+42",       "(U,B)",             (1/7)*(2/3)*(1/3)*2),
  c("84,42+42",       "(B,B)",             (1/7)*(1/3)*(1/3))
  ),
  nrow=23, ncol=3, byrow=TRUE)





topol.to.index <- function(topol) {
  w <- grep(topol, eighttip.topologies[,1], fixed=TRUE)
  stopifnot(length(w) == 1)
  w[1]
}


gettopol.probs.for.ops <- function(ops) {
  trees <- read.trees.remove.burnin(paste0(fpathbase, ops, ".txt"))
  inode.count <- trees[[1]]$Nnode
  stopifnot(inode.count == 7)
  counts <- rep(0,23)
  for (t in 1:length(trees)) {
    tree <- trees[[t]]
    po.edge <- reorder.phylo(tree, "postorder")$edge
    nodeinfo <- poedges.to.nodeinfo(po.edge, inode.count)
    topol <- nodeinfo.to.topology8(nodeinfo, 15, "")
    cat(topol)
    i <- topol.to.index(topol)
    counts[i] <- counts[i] + 1
    cat(" ", i, " #  ")
  }
  counts <- counts / sum(counts)
  counts
}


get.and.plot.topol.probs.for.ops <- function(ops) {
  counts <- gettopol.probs.for.ops(ops)
  par(mar=c(6, 4, 2, 2) + 0.1, cex.axis=0.7)
  barplot(counts, space=0, ylim=c(0,1.2*max(counts)),
          ylab="probability",
          names.arg=eighttip.topologies[,2], las=2, )
  mtext(text=paste0("Operators ", ops), line=0)
  points((1:23)-0.5, as.double(eighttip.topologies[,3]), pch=18, cex=1.5)
}


pdf(file=paste0(fpathbase, "topol.pdf"), width=7, height=7)
par(mfcol=c(2,2))
get.and.plot.topol.probs.for.ops("B")
get.and.plot.topol.probs.for.ops("BF")
get.and.plot.topol.probs.for.ops("BN")
get.and.plot.topol.probs.for.ops("BR")
dev.off()


pdf(file=paste0(fpathbase, "topolBR.pdf"), width=4, height=4)
get.and.plot.topol.probs.for.ops("BR")
dev.off()


get.and.make.pdf.node.heights.for.ops("B")
get.and.make.pdf.node.heights.for.ops("BF")
get.and.make.pdf.node.heights.for.ops("BN")
get.and.make.pdf.node.heights.for.ops("BR")

get.and.make.pdf.node.heights.for.ops.kth.speciation("BR", 2)
